/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef ABOUT_DIALOG
#define ABOUT_DIALOG

#include <akndialog.h>
#include "base/util/StringBuffer.h"
#include "CommonOptionsSettingList.h"

#define SCROLLING_STEP          10      // Vertical scrolling, in pixes
#define BOTTOM_WHITE_SPACE      10      // in pixels

using namespace Funambol;

// FORWARD DECLARATIONS
class CFunambolAppUi;

/**
 * The about dialog.
 * Contains the application logo, and labels with application info.
 */
class CAboutDialog : public CEikDialog
{
private:
    
    CFunambolAppUi* iAppUI;
    
    /** 
     * Y-position of the logical screen against the visible screen.
     * When dialog is loaded, its value is 0.
     * Then, scrolling down the dialog, it can be < 0.
     */
    TInt iPosition;
    
    /// The max (logical) dialog height.
    /// It's calculated at runtime, considering all visible objects.
    TInt iMaxDialogHeight;

    
    CAboutDialog(CFunambolAppUi* appUi);
    ~CAboutDialog();
    
    void ConstructL();
    
    /// Overrided to replace labels text at runtime. 
    void PreLayoutDynInitL();
    
    /// Overrided to force a white background. 
    void PostLayoutDynInitL();

    TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode /*aType*/);
    
    /**
     * Redraws all visible objects in the dialog, shifting them vertically by an offset.
     * If offset = 0, the dialog will just be refreshed.
     * @param offset  number of pixels to shift the dialog, vertically
     */
    void shiftObjects(const int offset);
    
    
public:
    
    ///Creates a new CAboutDialog object.
    static CAboutDialog* NewL(CFunambolAppUi* appUi);
    
    /// Redraws all visible objects in the dialog.
    void RefreshDialog();
    
};

#endif  // ABOUT_DIALOG

