/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


/*
 * Each item has an access count. An item cannot be fully deleted unless its access count is zero. 
 */

#ifndef CONTACTSENGINE_H_
#define CONTACTSENGINE_H_

#include <cntdb.h>
#include <cntdbobs.h>

#include "FunambolAppUi.h"

/*
 * Contacts engine.
 * Provide a function to delete all contacts. Link against 
 * cntmodel.lib. Requires ReadUserData,WriteUserData capabilities
 */
class CCleanupContactsManager: public CActive, public MContactDbObserver
{

public:

    /**
     * Two-phased constructor.
     */
    static CCleanupContactsManager* NewL(CFunambolAppUi* appUi);

    /**
     * Two-phased constructor.
     */
    static CCleanupContactsManager* NewLC(CFunambolAppUi* appUi);
    
    /**
     *  Destructor.
     */
    ~CCleanupContactsManager();
    
    /**
     * Delete all contacts.
     * Delete all contacts of default contacts database. Please
     * note that "Each item has an access count. An item cannot be fully deleted
     * unless its access count is zero."  
     */
    void DeleteAllEntriesL();
    
    /*
     * Stop DeleteAllContacts operation
     */
    void Stop(void);
    
    /**
     * Implements virtual method of MContactDbObserver.
     */
    void HandleDatabaseEventL(TContactDbObserverEvent aEvent);
    
        
private:

    /**
     * C++ default constructor can NOT contain any code, that might leave.
     */
    CCleanupContactsManager(CFunambolAppUi* appUi);

    /**
     * Symbian 2nd phase constructor can leave.
     */
    void ConstructL();
    
    /*
     * Required by CActive
     */
    void RunL(void);
    void DoCancel(void);

private:

    /** pointer to default contacts database */
    CContactDatabase*            iContactsDb;
    /** pointer to contacts change notifier */
    CContactChangeNotifier*      iNotifier;
    
    TContactItemId               iContactItemId;
    TContactIter*                iContactIter;
    TBool                        iCanceled;

    CFunambolAppUi*              appUi;
};

#endif /*CONTACTSENGINE_H_*/
