/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef CLIENTPUSH_H_
#define CLIENTPUSH_H_

#include <cntdef.h>
#include <CNTDBOBS.H>
#include <cntdb.h>
#include <calchangecallback.h>
#include <calsession.h>

#include "FunambolAppUi.h"

// The default CP interval, in seconds.
#define CP_INTERVAL             300     


// FORWARD DECLARATIONS
class CFunambolAppUi;

using namespace Funambol;

/**
 * Provides simple interface to Contacts and Calendar observers.
 * Link against cntmodel.lib, calinterimapi.lib
 * The new CalInterim API has been introduced with the Symbian 9.1 releases to replace the old Agenda Model API.
 *  Symbian OS v9.1  Symbian OS reference  C++ component reference  App engines CALINTERIMAPI  MCalChangeCallBack
 * 
 */
class CClientPush : public CBase, public MContactDbObserver, public MCalChangeCallBack2 
{
    
private:
    
    CFunambolAppUi* iAppUi;

    /// for contacts monitoring
    CContactDatabase* iContactDb;
    
    /// for calendar monitoring
    CCalSession* iCalSession;
    
    CContactChangeNotifier* iNotifier;
    
    // True if is currently listening to modifications.
    bool                    iListening;
    
    /// The filter used for calendar entries.
    /// It's created new in ConstructL().
    CCalChangeNotificationFilter* iFilter;
    
    /**
     * Perform the second phase construction of a CSymbianTransportAgent object.
     */
    void ConstructL(CFunambolAppUi* aAppUi);

    /**
     * Perform the first phase of two phase construction.
     */
    CClientPush();

  
public:
    
    /**
     * Create a CClientPush object.
     * @return A pointer to the created instance of CWebClientEngine
     */
    static CClientPush* NewL(CFunambolAppUi* aAppUi);

    /**
     * Create a CClientPush object.
     * @return A pointer to the created instance of CSymbianTransportAgent
     */
    static CClientPush* NewLC(CFunambolAppUi* aAppUi);

    /**
     * Destroy the object
     */
    ~CClientPush();
    
    
    /// Starts the observers for Contacts and Calendar+Events.
    void start();
    
    /// Stops the observers for Contacts and Calendar+Events.
    void stop();
    
    /// Returns true if CP is active (we're listening for notifications).
    bool isRunning() { return iListening; }

    
    /**
     * Observer that handles changes to a contact database.
     * Inherited from MContactDbObserver.
     * @param aEvent  contains informations about the contact changed
     *                aEvent.iType:      the type of event
     *                aEvent.iContactId: the id of the involved contact
     */
    void HandleDatabaseEventL(TContactDbObserverEvent aEvent);

    /**
     * Observer that handles changes to the model view (calendar & tasks).
     * Inherited from MCalChangeCallBack.
     * @param aChangeEntryType  the type of entry modified
     */
    void CalChangeNotification(RArray<TCalChangeEntry>& aChangeItems);

};


#endif /*CLIENTPUSH_H_*/
