/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef COMMON_OPTIONS_SETTING_LIST
#define COMMON_OPTIONS_SETTING_LIST

#include <akndialog.h> 
#include <aknlists.h>
#include <aknsettingitemlist.h> 
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "Funambol.hrh"
#include "commonDefs.h"

using namespace Funambol;


/**
 * This class is used to rapresent the setting list of controls
 * in all options dialogs (emailOptions / PIMOptions / SyncMethod).
 */
class CCommonOptionsSettingList : public CAknSettingItemList
{
    
private:
    
    TBuf<MAX_USERNAME_LEN> iUsername;
    TBuf<MAX_PASSWORD_LEN> iPassword;
    TBuf<MAX_SYNCURL_LEN>  iSyncURL;
    
    TBool   iSyncEnabled;
    TBool   iAutostart;
    TInt    iSyncMode;
    TInt    iSyncUsing;
    TInt    iSyncInterval;
    TInt    iAPNSelection;
    TInt    iLogLevel;

    TBool   iCleanupEmailEnabled;
    TBool   iCleanupContactEnabled;
    TBool   iCleanupCalendarEnabled;
    TBool   iCleanupNotesEnabled;

private:
    
    static const TBuf<1> emptyText;
    
    /// Returns the settingItem from its ID. NULL if not found.
    CAknSettingItem* findItem(const TSettingItems aItemID);
    
    /**
     * Returns the global index from the visible index.
     * global index = index of an item, considering all items in the list
     * visible index = index of an item, considering only visible items
     */
    TInt GetGlobalItemsIndex(const TInt index);
    
public:
    
    CCommonOptionsSettingList();
    
    CAknSettingItem* CreateSettingItemL(TInt aIdentifier);
    
    /// Shows the popup dialog for the current selected item.
    void EditCurrentItemL();
    
    /**
     * Hide/show the desired item with ID  = aItemID.
     * @param aItemID   ID of the desired item
     * @param aHide     ETrue to hide the item, EFalse to show it
     */
    void HideItem(const TSettingItems aItemID, const TBool aHide);
    
    /**
     * Shows a "*" symbol near the desired item.
     * @param aItemID   ID of the desired item
     */
    void SetMandatoryItem(const TSettingItems aItemID);
    
    /// Refresh the desired item with ID  = aItemID.
    void RefreshItem(const TSettingItems aItemID);

    /**
     * This method obtains the current value's setting text, for the item 'aItemID'.
     * @return Reference to descriptor with the text 
     */
    const TDesC& GetItemText(const TSettingItems aItemID);

    /// Creates and disables the vertical scrollbar.
    void initScrollbar();
    
    /**
     * Creates dynamically the APN-selection item, as a list
     * of choices with the current IAP names.
     * @param defaultAPNText  if not NULL, the text to display for the default option (usually = "Always ask") 
     *                        It's a new allocated buffer that will be handled inside this method.
     */
    void ConstructAPNList(HBufC* defaultAPNText = NULL);
    
    
    //
    // ------ get/set methods ------
    //
    StringBuffer getUsername()        { return bufToStringBuffer(iUsername); }
    StringBuffer getPassword()        { return bufToStringBuffer(iPassword); }
    StringBuffer getSyncURL()         { return bufToStringBuffer(iSyncURL);  }
    TSyncUsing   getSyncUsing()       { return (TSyncUsing)iSyncUsing;       }
    TBool        getSyncEnabled()     { return iSyncEnabled;                 }
    TBool        getAutostart()       { return iAutostart;                   }
    TInt         getSyncMode()        { return iSyncMode;                    }
    TInt         getSyncInterval()    { return iSyncInterval;                }
    TInt         getAPNSelection()    { return iAPNSelection;                }
    TBool        getEmailEnabled()    { return iCleanupEmailEnabled;         }
    TBool        getContactEnabled()  { return iCleanupContactEnabled;       }
    TBool        getCalendarEnabled() { return iCleanupCalendarEnabled;      }
    TBool        getNotesEnabled()    { return iCleanupNotesEnabled;         }
    TInt         getLogLevel()        { return iLogLevel;                    }
    
    
    void setUsername(const StringBuffer& value);
    void setPassword(const StringBuffer& value);
    void setSyncURL (const StringBuffer& value);
    void setSyncUsing   (const TSyncUsing aSyncUsing) { iSyncUsing      = aSyncUsing;    }
    void setSyncEnabled (const TBool aSyncEnabled)    { iSyncEnabled    = aSyncEnabled;  }
    void setAutostart   (const TBool aAutostart)      { iAutostart      = aAutostart;    }
    void setSyncMode    (const TInt aSyncMode)        { iSyncMode       = aSyncMode;     }
    void setSyncInterval(const TInt aSyncInterval)    { iSyncInterval   = aSyncInterval; }
    void setAPNSelection(const TInt aAPNSelection)    { iAPNSelection   = aAPNSelection; }
    void setLogLevel    (const TInt aLogLevel)        { iLogLevel       = aLogLevel;     }
};




/**
 * Extends the CAknBinaryPopupSettingItem, for a binary setting item.
 * EditItemL() is overrided, to avoid displaying the popup setting page.
 */
class CEasyBinaryPopupSettingItem : public CAknBinaryPopupSettingItem
{ 

public:
    CEasyBinaryPopupSettingItem(TInt aIdentifier, TBool& aBinaryValue ) 
                                  : CAknBinaryPopupSettingItem(aIdentifier, aBinaryValue) {}

    /// Overrided, to avoid displaying the popup setting page.
    /// The binary value is just inverted.
    virtual void EditItemL( TBool aCalledFromMenu );
};



#endif  // COMMON_OPTIONS_SETTING_LIST

