/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef SYNC_ACTIVE_OBJECT_INC
#define SYNC_ACTIVE_OBJECT_INC

#include <eikenv.h>
#include "base/util/StringBuffer.h"

#include "commonDefs.h"
#include "customization.h"

// Forward class declaration
class CSyncEngine;
class TimeoutTimer;

/**
 * This class allows executing a sync in a separate ActiveObject with a timeout.
 * It is meant to be used only by the SyncEngine.
 */

class SyncActiveObject : public CActive
{

private:

    enum State {
        EDone,
        ECancelled,
        ESync,
        EStopped,
        ENone
    };

    StringBuffer    iCacheFile;
    TInt            status;
    RTimer          iTimer;
    
    static State             iState;
    static CSyncEngine*      iSyncEngine;
    static SyncActiveObject* syncAO;
    static TimeoutTimer*     tt;

public:
    static void syncConfigLD(CSyncEngine* aSyncEngine, StringBuffer cacheFile);
    static bool CheckSyncInProgress();

private:
    static SyncActiveObject* NewL(StringBuffer cacheFile);
    static SyncActiveObject* NewLC(StringBuffer cacheFile);
    ~SyncActiveObject();

    SyncActiveObject(StringBuffer cacheFile);
    void ConstructL();
    void syncL();
    void RunL();
    void Stop();
    void sync();
    static void syncConfigEnded(bool timeout, TInt status);

protected:
    void DoCancel();

    friend class TimeoutTimer;
};

#endif

