/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef INCL_SYMBIAN_CLIENT_UTILS
#define INCL_SYMBIAN_CLIENT_UTILS

#include <gulicon.h>
#include <akniconutils.h>
#include "base/fscapi.h"
#include "base/util/StringBuffer.h"

using namespace Funambol;

TInt         getSourceIDFromName(const StringBuffer& name);
StringBuffer getSourceNameFromID(const TInt     aSourceID);

// Used for scheduler option list.
TInt getSchedulerIndexFromMinutes     (const TInt minutes);
TInt getSchedulerResourceIDFromMinutes(const TInt minutes);
TInt getMinutesFromSchedulerIndex     (const TInt index);
TInt getSchedulerResourceIDFromIndex  (const TInt index);

/**
 * Used by SyncEngine::syncConfig().
 * If a connection is not already active, we start a new one (using FConnection).
 * In case of errors, retry 'maxNumRetries' times, asking the user for the APN.
 * @param maxNumRetries  max number of times we can ask the user to select a new APN
 * @return               true if connected, false if could not connect
 */
bool startConnection(const TInt maxNumRetries);

/**
 * Loads an icon and appends it into an array of icons 'icons'.
 * @param icons  [OUT] the array of icons, to append the new icon
 * @param bmpID  the identifier of the icon (mbm or mif)
 * @param maskID the identifier of the mask, optional
 */
void appendIcon(CArrayPtr<CGulIcon>* icons, const TInt bmpID, TInt maskID = 0);

/**
 * Returns an integer value rapresenting the build number read from
 * the string version passed (e.g. "6.1.12" -> 60112). 
 */
int getBuildNumberFromVersion(const char* swv);

/**
 * Merges sourceArray into destArray.
 * Every value ont found in destArray is copied.
 * @return the new number of items in destArray
 */
int mergeRArray(RArray<TInt>& destArray, const RArray<TInt>& sourceArray);


#endif /*INCL_SYMBIAN_CLIENT_UTILS*/
