/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <eikspace.h> 
#include "uid3.h"

#include "base/Log.h"
#include "base/util/stringUtils.h"
#include "commonDefs.h"
#include "customization.h"
#include "Funambol.hrh"
#include "ClientSettings.h"
#include "FunambolAppUi.h"
#include "AboutDialog.h"

using namespace Funambol;


CAboutDialog* CAboutDialog::NewL(CFunambolAppUi* appUi)
{
    CAboutDialog* self = new (ELeave)CAboutDialog(appUi);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    return self;
}

CAboutDialog::CAboutDialog(CFunambolAppUi* appUi) : iAppUI(appUi) {}

CAboutDialog::~CAboutDialog() {}

void CAboutDialog::ConstructL()
{   

}


void CAboutDialog::PreLayoutDynInitL()
{
    //LOG.debug("CAboutDialog::PreLayoutDynInitL");
    // TODO: use Position() and SetPosition() to separate labels (now using "\n")

    //
    // Set the text for the first label:
    // 
    if ( ControlOrNull(EAboutAppInfo) )
    {
        StringBuffer text(APPLICATION_NAME);
        text += "\nVersion ";
        text += ClientSettings::getInstance()->getSwv();
        
        if (PRE_ALPHA_RELEASE)  { text += "\n(pre-alpha)"; } 
        else if (ALPHA_RELEASE) { text += "\n(alpha)";     }
        else if (BETA_RELEASE)  { text += "\n(beta)";      }
        text += "\n \n ";

        if (ENABLE_ABOUT_MAIN_WEB_SITE) {
            text += ABOUT_COPYRIGHT;
            text += "\n ";
            text += iAppUI->readStringResource(R_ABOUT_MAIN_WEB_SITE);
            //text += "\n ";
        }

        CEikLabel* label = static_cast<CEikLabel*>( ControlOrNull(EAboutAppInfo) );
        HBufC* labelText = stringBufferToNewBuf(text);
        label->SetTextL(*labelText);
        delete labelText;
        
        label->SetFont(LatinBold19());
        
        //label->OverrideColorL( EColorLabelText, KRgbRed );
        //label->SetEmphasis( CEikLabel::EPartialEmphasis );  
    }

    //
    // Set the text for the second label
    //
    if ( ControlOrNull(EAboutPoweredBy) )
    {
        CEikLabel* label = static_cast<CEikLabel*>( ControlOrNull(EAboutPoweredBy) );
        
        if (ENABLE_ABOUT_POWERED_BY) {
            StringBuffer text = iAppUI->readStringResource(R_ABOUT_POWERED_BY);
            text += iAppUI->readStringResource(R_ABOUT_POWERED_WEB_SITE);
            text += "\n ";
            
            HBufC* labelText = stringBufferToNewBuf(text);
            label->SetTextL(*labelText);
            delete labelText;
            
            label->SetFont(LatinBold16());
        }
        else {
            label->SetTextL(_L(""));
        }
    }
    
    //
    // Set the text copyright license
    //
    if ( ControlOrNull(EAboutCopyright) )
    {
        CEikLabel* label = static_cast<CEikLabel*>( ControlOrNull(EAboutCopyright) );
        if (ENABLE_ABOUT_COPYRIGHT) {
            // TODO: make font smaller (http://www.newlc.com/topic-15357)
            // TODO: add a scrollbar!  (http://wiki.forum.nokia.com/index.php/CS000869_-_Custom_control:_Scrollbars)
            HBufC* labelText = iEikonEnv->AllocReadResourceLC(R_ABOUT_DIALOG_TEXT);
            label->SetTextL(*labelText);
            CleanupStack::PopAndDestroy();      // labelText
            
            label->SetFont(LatinBold13());
        }
        else {
            label->SetTextL(_L(""));
        }
    }

    CEikDialog::PreLayoutDynInitL();
}



void CAboutDialog::PostLayoutDynInitL()
{
	//LOG.debug("CAboutDialog::PostLayoutDynInitL");
    CEikDialog::PostLayoutDynInitL();
    
    //
    // Set the background to white
    //
    if ( ControlOrNull(EAboutBkground) )
    {
        CEikSpacer* bg = static_cast<CEikSpacer*>( ControlOrNull(EAboutBkground) );
        bg->SetRect( Rect() );
        bg->SetPosition( TPoint(0,0) );
        bg->SetHeight( Size().iHeight );
        bg->SetWidth( Size().iWidth );
        bg->SetColor( KRgbWhite );
        bg->SetClear( EFalse );
    }
    
    // Initial dialog position = 0.
    iPosition = 0;
    
    // Set the max logical height of the dialog to the screen height.
    iMaxDialogHeight = Size().iHeight;

    // Find the last object, and set iMaxDialogHeight to its bottom position.
    // This must be done after the object have been drawn, so we do it in PostLayoutDynInitL().
    if (ENABLE_ABOUT_COPYRIGHT) {
        CCoeControl* control = ControlOrNull(EAboutCopyright);
        if (control) {
            TInt pos = control->Position().iY + control->Size().iHeight;
            if (pos > iMaxDialogHeight) {
                iMaxDialogHeight = pos;
                LOG.debug("iMaxDialogHeight = %d", iMaxDialogHeight);
            }
        }
    }
    else if (ENABLE_ABOUT_POWERED_BY) {
        CCoeControl* control = ControlOrNull(EAboutPoweredBy);
        if (control) {
            TInt pos = control->Position().iY + control->Size().iHeight;
            if (pos > iMaxDialogHeight) {
                iMaxDialogHeight = pos;
                //LOG.debug("iMaxDialogHeight = %d", iMaxDialogHeight);
            }
        }
    }

    //TRAPD(error, OverrideColorL(EColorControlBackground, KRgbWhite));
}



TKeyResponse CAboutDialog::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode /*aType*/)
{
    //LOG.debug("CAboutDialog::OfferKeyEventL - code 0x%04x", aKeyEvent.iCode);
    
    // Move focus
    switch (aKeyEvent.iCode)
    {
        case EKeyUpArrow:
        {
            shiftObjects(SCROLLING_STEP);   // move down
            break;
        }
        case EKeyDownArrow:
        {
            shiftObjects(-SCROLLING_STEP);  // move up
            break;
        }
        default:
        {
            break;
        }
    }
    
    // Note: we MUST always return 'key consumed' to avoid this key is
    // captured by other dialogs, under this one.
    return EKeyWasConsumed;
}


void CAboutDialog::shiftObjects(const int offset)
{
    //LOG.debug("CAboutDialog::shiftObjects by %d (iPos = %d)", offset, iPosition);
    
    // Calculate the max position allowed.
    TInt maxPosition = (iMaxDialogHeight - Size().iHeight) + BOTTOM_WHITE_SPACE;
    
    if ((offset > 0) && (iPosition >= 0)) {
        return;  // Lock on the top of the dialog.
    }
    if ((offset < 0) && (iPosition <= -maxPosition)) {
        return;  // Lock on the bottom of the dialog.
    }
    

    // Shift all objects.
    // - Get single controls with ControlOrNull (don't use ComponentControl(), it won't work correctly).
    // - Redraw single controls one by one, background must be redrawn for first.
    CCoeControl* control = NULL;
    control = ControlOrNull(EAboutBkground);
    if (control) {
        CEikSpacer* bg = static_cast<CEikSpacer*>(control);
        bg->SetRect( Rect() );
        bg->SetPosition( TPoint(0,0) );
        bg->SetHeight( Size().iHeight );
        bg->SetWidth( Size().iWidth );
        bg->SetColor( KRgbWhite );
        bg->SetClear( EFalse );
        bg->DrawNow();
    }
    control = ControlOrNull(EAboutLogo);
    if (control) {
        TPoint pos = control->Position();
        pos.iY += offset;
        control->SetPosition(pos);
        control->DrawNow();
    }
    control = ControlOrNull(EAboutAppInfo);
    if (control) {
        TPoint pos = control->Position();
        pos.iY += offset;
        control->SetPosition(pos);
        control->DrawNow();
    }
    control = ControlOrNull(EAboutPoweredBy);
    if (control) {
        TPoint pos = control->Position();
        pos.iY += offset;
        control->SetPosition(pos);
        control->DrawNow();
    }
    control = ControlOrNull(EAboutCopyright);
    if (control) {
        TPoint pos = control->Position();
        pos.iY += offset;
        control->SetPosition(pos);
        control->DrawNow();
    }
    
    // Update screen position.
    iPosition += offset;

    return;
}

void CAboutDialog::RefreshDialog()
{
    // Will redraw all objects, in the right order.
    shiftObjects(0);
}

