/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <akntitle.h>
#include <barsread.h>
#include "uid3.h"

#include "base/SymbianLog.h"
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "base/util/symbianUtils.h"

#include "customization.h"
#include "commonDefs.h"
#include "Funambol.hrh"
#include "CleanupDialog.h"
#include "FunambolAppUI.h"

using namespace Funambol;


// Cleanup removes all of your devices's Calendar. Continue?

CCleanupDialog* CCleanupDialog::NewL(TInt aMenuTitleResourceId,
                                     CFunambolAppUi* aAppUI)
{
    CCleanupDialog* self = new (ELeave)CCleanupDialog();
    CleanupStack::PushL(self);
    self->ConstructL(aMenuTitleResourceId, aAppUI);
    CleanupStack::Pop(self);
    return self;
}

void CCleanupDialog::ConstructL(TInt aMenuTitleResourceId,
                                CFunambolAppUi* aAppUI)
{
    iAppUI       = aAppUI;
    iSettingList = NULL;
    CAknDialog::ConstructL(aMenuTitleResourceId);
}

CCleanupDialog::CCleanupDialog() {}


CCleanupDialog::~CCleanupDialog()
{
    if (iSettingList) {
        delete iSettingList;
        iSettingList = NULL;
    }
}

void CCleanupDialog::PreLayoutDynInitL()
{
    LOG.debug("CleanupDialog::PreLayoutDynInitL");

    if (!iSettingList) // if we haven't already created iSettingList
    {
        // construct control and set parent
        iSettingList = new (ELeave) CCommonOptionsSettingList();
        iSettingList->SetMopParent(this);

        // construct control based on resource definition
        TResourceReader reader;

        // CreateResourceReaderLC will allocate a buffer to be used by
        // the TResourceReader. This buffer is pushed onto the cleanup
        // stack - not the TResourceReader itself
        iCoeEnv->CreateResourceReaderLC(reader, R_SOURCES_CLEANUP_SETTING_LIST);
        iSettingList->ConstructFromResourceL(reader);

        // Clean up the buffer allocated above, NOT the reader itself.
        // Cannot use expected item overload of PopAndDestroy() as buffer 
        // is inaccessible. 
        CleanupStack::PopAndDestroy();
        
        // Removes the vertical scrollbar
        iSettingList->initScrollbar();

        // If mail/notes are disabled then remove the item
        if (!ENABLE_SOURCE_MAIL) {
            iSettingList->HideItem(ESettingsListCleanupEmail, ETrue);
        }
        // TODO: uncomment when Notes Cleanup is done
        //if (!ENABLE_SOURCE_NOTES) {
            iSettingList->HideItem(ESettingsListCleanupNotes, ETrue);
        //}

        // Activate the settings list
        iSettingList->ActivateL();
    }
    
    iAppUI->changeAppTitle(R_STRING_CLEANUP_TITLE);
    
    CAknDialog::PreLayoutDynInitL();
}

HBufC* CCleanupDialog::PrepareConfirmMessageLC(TBool sources[4]) {
    HBufC* msg = iEikonEnv->AllocReadResourceLC(R_STRING_CONFIRM_CLEANUP);

    HBufC* labelEmail    = iEikonEnv->AllocReadResourceLC(R_STRING_LABEL_EMAIL);
    HBufC* labelContacts = iEikonEnv->AllocReadResourceLC(R_STRING_LABEL_CONTACTS);
    HBufC* labelCalendar = iEikonEnv->AllocReadResourceLC(R_STRING_LABEL_CALENDAR);
    HBufC* labelNotes    = iEikonEnv->AllocReadResourceLC(R_STRING_LABEL_NOTES);
    HBufC* labels[4]     = { labelEmail, labelContacts, labelCalendar, labelNotes };

    RBuf message;
    TInt maxLen = msg->Size() + labelEmail->Size() + labelContacts->Size()
                              + labelCalendar->Size() + labelNotes->Size();
    // Increase the size to take into account separators (commas, "and"...)
    maxLen += 32;
    message.Create(maxLen);

    message.Append(*msg);

    // Count how many sources we have to clean, so we know if and where the
    // "and" must be positioned in the message
    TInt numSources = 0;
    for(TInt i=0;i<4;++i) {
        if (sources[i]) {
            numSources++;
        }
    }
    // Now prepare the msg
    TInt idx = 0;
    for(TInt i=0;i<4;++i) {
        if (sources[i]) {
            if (idx == numSources - 1 && numSources > 1) {
                // just before the last one we add the "and"
                HBufC* andString = iEikonEnv->AllocReadResourceLC(R_STRING_AND);
                message.Append(' ');
                message.Append(*andString);
                message.Append(' ');
                CleanupStack::PopAndDestroy();
            } else if (idx > 0) {
                message.Append(',');
                message.Append(' ');
                // append the comma
            }
            // Append label[i] to msg
            message.Append(*(labels[i]));
            idx++;
        }
    }
    HBufC* continueMsg = iEikonEnv->AllocReadResourceLC(R_STRING_CONTINUE);
    message.Append('.');
    message.Append(' ');
    message.Append(*continueMsg);
    message.Append('?');
    CleanupStack::PopAndDestroy(6);
    HBufC* res = message.Alloc();
    CleanupStack::PushL(res);
    message.Close();
    return res;
}

TBool CCleanupDialog::OkToExitL(TInt aButtonId)
{
    //LOG.debug("CPIMOptionsDialog::OkToExitL: button %d", aButtonId);
    
    // Returning ETrue causes the form to exit
    switch (aButtonId)
    {
        case EAknSoftkeyBack:
        {
            return ETrue;
        }
        case ECleanupContinue:      
        {
            TBool sources[4];
            sources[0] = iSettingList->getEmailEnabled();
            sources[1] = iSettingList->getContactEnabled();
            sources[2] = iSettingList->getCalendarEnabled();
            sources[3] = iSettingList->getNotesEnabled();

            TBool atLeastOne = EFalse;
            for(TInt i=0;i<4;++i) {
                if (sources[i]) {
                    atLeastOne = ETrue;
                    break;
                }
            }
            if (!atLeastOne) {
                // In this case we do not quit and do anything.
                // The user can go back via the back button
                return EFalse;
            }

            HBufC* title = charToNewBuf("");
            CleanupStack::PushL(title);
            HBufC* msg = PrepareConfirmMessageLC(sources);
            // Display the query dialog.
            //TBool toCleanup = iEikonEnv->QueryWinL(resourceID);
            TBool toCleanup = iEikonEnv->QueryWinL(*title, *msg);
            CleanupStack::PopAndDestroy(2); // msg and title
            if (toCleanup) {
                // The user selected "Yes": exit.
                iAppUI->cleanup(iSettingList->getEmailEnabled(),
                                iSettingList->getContactEnabled(),
                                iSettingList->getCalendarEnabled(),
                                iSettingList->getNotesEnabled());
                return ETrue;
            }
            break;
        }
        // Not used...
        /*case EEikCmdExit:
        case EAknSoftkeyExit:
        {
            iAppUI->Exit();
            break;
        }*/
        default:
        {
            // Other commands forwarded to CAknDialog.
            return CAknDialog::OkToExitL(aButtonId);  
        }
    }
    return EFalse;
}


TKeyResponse CCleanupDialog::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType != EEventKey) {
        return EKeyWasNotConsumed;
    }
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3)          // ???????? why this code ????????
    {
        iSettingList->EditCurrentItemL();
        return EKeyWasConsumed;
    }
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to setting list
        case EKeyDownArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}

