/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <e32base.h>
#include <e32cmn.h>
#include <msvapi.h>

#include "base/Log.h"

#include "FunambolAppUi.h"
#include "CleanupMailManager.h"
#include "Customization.h"
#include "ClientSettings.h"

#ifdef FAKE_IMAP_ACCOUNT
#include "FakeImapAccountManager.h"
#else
#include "ImapAccountManager.h"
#endif


/**
 * CCleanupMailManager::NewL()
 * Two-phased constructor.
 */
CCleanupMailManager* CCleanupMailManager::NewL(CFunambolAppUi* aAppUi)
{
    CCleanupMailManager* self = CCleanupMailManager::NewLC(aAppUi);
    CleanupStack::Pop(self);
    return self;
}

/**
 * CCleanupMailManager::NewLC()
 * Two-phased constructor.
 */
CCleanupMailManager* CCleanupMailManager::NewLC(CFunambolAppUi* aAppUi)
{
    CCleanupMailManager* self = new(ELeave) CCleanupMailManager(aAppUi);
    self->ConstructL();
    CleanupStack::PushL(self);
    return self;
}

/**
 * CCleanupMailManager::CCleanupMailManager()
 * C++ default constructor can NOT contain any code, that might leave.
 */
CCleanupMailManager::CCleanupMailManager(CFunambolAppUi* aAppUi)
                                        : CActive(CActive::EPriorityStandard),
                                        iAppUi(aAppUi)
{
    iDone = EFalse;
    iCanceled = EFalse;
}

/**
 * CCleanupMailManager::ConstructL()
 * Symbian 2nd phase constructor can leave.
 */
void CCleanupMailManager::ConstructL()
{
    CActiveScheduler::Add(this);
}

/**
 * CCleanupMailManager::~CCleanupMailManager()
 * Class destructor
 */
CCleanupMailManager::~CCleanupMailManager()
{
    if( IsActive() ) {
        Cancel();
    }
    
}

/**
 * CCleanupMailManager::DeleteAllMailsL()
 */
void CCleanupMailManager::DeleteAllMailsL(void)
{
    TRequestStatus* status = &iStatus;
    User::RequestComplete(status, KErrNone);
    SetActive();
}

/**
 * CCleanupMailManager::ReallyDeleteAllMailsL()
 */
void CCleanupMailManager::ReallyDeleteAllMailsL(void)
{
    LOG.info("Cleaning up all Emails...");
    
    // Deleting all entries into mailbox seems not to work
    // as expected/desired
    // As long as we don't find a better solution, we 
    // delete the mailbox and re-create it
    //LOG.debug("ReallyDeleteAllMailsL");

    const StringBuffer profileSb(SYNC_PROFILE_NAME);
    HBufC* profile = stringBufferToNewBuf(profileSb);
    CleanupStack::PushL(profile);

    // Retrieve original values
    ClientSettings* config = ClientSettings::getInstance();
    config->open();
    if (!config->read()) { // is it necessary?
        LOG.error("Cannot read configuration");
        return;
    }
 
    // delete imap account
    CImapAccountManager* accountManager = CImapAccountManager::NewL();
    accountManager->DeleteAccountL(*profile);
    
    // create again imap account
    accountManager->setCredentials(config->getAccessConfig().getUsername(),
                                   config->getAccessConfig().getPassword());
    accountManager->setDisplayName(config->getDisplayName());
    accountManager->setAddress(config->getEmailAddress());
    accountManager->setIap(config->getIapName());
    
    delete accountManager;
    
    CleanupStack::PopAndDestroy(profile);
    
    iDone = ETrue;
    
    TRequestStatus* status = &iStatus;
    User::RequestComplete(status, KErrNone);
    SetActive();
}

void CCleanupMailManager::DoCancel(void)
{
    iCanceled = ETrue;
}


void CCleanupMailManager::RunL(void)
{
    if(iCanceled){
        // operation has been stopped
        iAppUi->mailCleanupCompleted();
    }
    else if(iDone){
        // operation completed
        iAppUi->mailCleanupCompleted();
    }
    else {
        ReallyDeleteAllMailsL();
    }
}

