/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <cntdb.h>
#include <e32std.h>

#include "base/SymbianLog.h"
#include "ClientPush.h"
#include "FunambolAppUi.h"
#include "commonDefs.h"

using namespace Funambol;


CClientPush* CClientPush::NewL(CFunambolAppUi* aAppUi)
{
	CClientPush* self = CClientPush::NewLC(aAppUi);
    CleanupStack::Pop( self );
    return self;
}

CClientPush* CClientPush::NewLC(CFunambolAppUi* aAppUi)
{
    CClientPush* self = new (ELeave) CClientPush();
    CleanupStack::PushL( self );
    self->ConstructL(aAppUi);
    return self;
}

CClientPush::CClientPush() 
{
    iAppUi      = NULL;
    iContactDb  = NULL;
    iNotifier   = NULL;
    iCalSession = NULL;
    iListening  = false;
    iFilter     = NULL;
}

CClientPush::~CClientPush()
{
    if (iNotifier)   { delete iNotifier;   }
	if (iCalSession) { delete iCalSession; }
	if (iFilter)     { delete iFilter;     }
}


void CClientPush::ConstructL(CFunambolAppUi* aAppUi)
{
    iAppUi = aAppUi;
    
    LOG.debug("CP: Initializing ClientPush...");
	
	// Contacts monitoring: Open Contact DB
	iContactDb = CContactDatabase::OpenL();
	
	// Calendar monitoring
	// Symbian OS v9.1  Symbian OS reference  C++ component reference  App engines CALINTERIMAPI  CCalSession
	iCalSession = CCalSession::NewL();
	iCalSession->OpenL(KNullDesC);
	
    // Set up the filter for calendar: [now - 10years ; now + 10years]
    TTime now;
    now.HomeTime();
    TTime startTime = now - TTimeIntervalYears(10);
    TTime endTime   = now + TTimeIntervalYears(10);
    
    TCalTime startT, endT;
    startT.SetTimeLocalL(startTime);
    endT.SetTimeLocalL(endTime);
    CalCommon::TCalTimeRange filterTime(startT, endT);
    iFilter = CCalChangeNotificationFilter::NewL(EChangeEntryAll, ETrue, filterTime);
}
	

void CClientPush::start()
{
    // ---- Contacts observer ----
    if (iNotifier) { delete iNotifier; }
    LOG.info("CP: Start listening for contacts & calendar notifications");
    iNotifier = CContactChangeNotifier::NewL(*iContactDb, this);
    
    // ---- Calendar & Tasks observer ----
	iCalSession->StartChangeNotification(*this, *iFilter);
    
    iListening = true;
}

void CClientPush::stop()
{
    // ---- Contacts observer ----
    if (iNotifier) {
        LOG.info("CP: Stop listening for contacts notifications");
        delete iNotifier;
        iNotifier = NULL;
    }
    
    // ---- Calendar & Tasks observer ----
    LOG.info("CP: Stop listening for calendar notifications");
    iCalSession->StopChangeNotification();
    
    // This is necessary to discard future sync already scheduled.
    iAppUi->disableSyncScheduler();
    
    iListening = false;
}


void CClientPush::HandleDatabaseEventL(TContactDbObserverEvent aEvent)
{
    //StringBuffer tmp;
    //tmp.sprintf("CP: Contact item changed! action = %d", aEvent);
    //msgBox(tmp.c_str());
    
	switch (aEvent.iType)
	{
		case EContactDbObserverEventContactChanged:
		{
		    LOG.info("CP: Contact modified! ID = %d", aEvent.iContactId);
			iAppUi->StartSync(SOURCE_CONTACTS, ESyncTriggeredClientPush);
			break;
		}
		case EContactDbObserverEventContactDeleted:
		{
		    LOG.info("CP: Contact deleted! ID = %d", aEvent.iContactId);
			iAppUi->StartSync(SOURCE_CONTACTS, ESyncTriggeredClientPush);
			break;
		}
		case EContactDbObserverEventContactAdded:
		{
		    LOG.info("CP: Contact added! ID = %d", aEvent.iContactId);
			iAppUi->StartSync(SOURCE_CONTACTS, ESyncTriggeredClientPush);
			break;
		}
		default:
		    // Other events are ignored (see enum TContactDbObserverEventType)
		   	break;
	}
}


void CClientPush::CalChangeNotification(RArray<TCalChangeEntry>& aChangeItems)
{
    TInt num = aChangeItems.Count();
    
    //StringBuffer tmp;
    //tmp.sprintf("CP: Calendar item changed! num = %d", num);
    //msgBox(tmp.c_str());
    
    LOG.info("CP: Calendar item changed!");
	iAppUi->StartSync(SOURCE_CALENDAR, ESyncTriggeredClientPush);
}

