/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include <avkon.hrh>
#include <aknmessagequerydialog.h>
#include <aknnotewrappers.h>
#include <stringloader.h>
#include <f32file.h>
#include <s32file.h>
#include <hlplch.h>
#include <eikmenup.h>
#include <w32std.h>                 // for red key
#include <apgtask.h>                // for going background
#include <apgcli.h>                 // RApaLsSession
#include <bautils.h>                // jad file
#include <aknwaitdialog.h>          // wait note
#include <aknviewappui.h>           // red key 
#include <akntitle.h>               // to change app title
#include <eikbtgpc.h>               // to change main menu buttons
#include <akncontext.h>             // main image handling
#include <syncmlerr.h>
#include <apacmdln.h>               // for command line handling

#include "base/util/symbianUtils.h"
#include "base/adapter/PlatformAdapter.h"
#include "base/util/stringUtils.h"
#include "base/SymbianLog.h"
#include "spdm/DeviceManagementNode.h"
#include "client/DMTClientConfig.h"
#include "base/FConnection.h"

#include "uid3.h"
#include "Funambol.hrh"
#include "Funambol.pan"
#include "FunambolApplication.h"
#include "FunambolAppView.h"
#include "FunambolDocument.h"
#include "AccountSettingsDialog.h"
#include "CleanupDialog.h"
#include "Model.h"
#include "AOSync.h"
#include "ClientSettings.h"
#include "commonDefs.h"
#include "customization.h"
#include "PushEngine.h"
#ifdef __GCCE__
#include "SyncEngine.h"
#else 
#include "FakeSyncEngine.h"
#endif
#include "ClientPush.h"
#include "CleanupContactsManager.h"
#include "CleanupCalendarManager.h"
#include "CleanupMailManager.h"
#ifdef __GCCE__
#include "SyncProfileManager.h"
#else
#include "FakeSyncProfileManager.h"
#endif
#include "ProgressDialog.h"
#include "UpdateManager.h"
#include "utils.h"
#include "FunambolAppUi.h"

#ifdef FAKE_IMAP_ACCOUNT
#include "FakeImapAccountManager.h"
#else
#include "ImapAccountManager.h"
#endif

using namespace Funambol;


// C++ default constructor can NOT contain any code, that might leave.
CFunambolAppUi::CFunambolAppUi() :  CActive(CActive::EPriorityStandard),
                                    iAppView(NULL),
                                    iModel(NULL),
                                    iWaitDialog(NULL),
                                    iTimer(NULL),
                                    iSyncEngine(NULL),
                                    iPushEngine(NULL),
                                    iLastConfigSyncResult(405),
                                    contactsManager(NULL),
                                    calendarManager(NULL),
                                    cleanupWaitDlg(NULL),
                                    iRequestedMsgID(0),
                                    iRequestedWarningLevel(EInfoLevel),
                                    iAboutDialog(NULL),
                                    iAccountSettingsDialog(NULL)
{
}

// Destructor.
CFunambolAppUi::~CFunambolAppUi()
{
    delete iSyncEngine;
    delete iPushEngine;
    
    if (iAppView) {
        RemoveFromStack(iAppView);      // the control stack
        delete iAppView;
    }
}


void CFunambolAppUi::ConstructL()
{
    // The model MUST be initialized before ProcessCommandParametersL()
    CFunambolDocument* document = static_cast<CFunambolDocument*>(Document()); 
    iModel = document->Model();
    
    // Initialise app UI with standard value.
    BaseConstructL(CAknAppUi::EAknEnableSkin);
    
    // Replace the default main icon (aif image) with the bitmap+mask
    // from funambol.mbm resource. So we can have transparency ;)
    loadAppIcon();
}


TBool CFunambolAppUi::ProcessCommandParametersL( CApaCommandLine &aCommandLine )
{
    if(aCommandLine.OpaqueData().Length() > 0) {
        // Application opened manually.
        iModel->AutoStart() = EFalse;
    } 
    else {
        // Application opened by autostart.
        iModel->AutoStart() = ETrue;

        // Further check if autostart is enabled. If not, exit now!
        if ( !iModel->AutoStartEnable() ) {
            CAknAppUi::Exit();
        }
    }

    //
    // Init Log, Config and all the stuff.
    //
    InitializeClient();

    return CEikAppUi::ProcessCommandParametersL(aCommandLine);
}

void CFunambolAppUi::InitializeClient()
{
    TInt err = KErrNone;
    bool firstRun = false;
    
    PlatformAdapter::init(APPLICATION_URI);

    //
    // First load of the configuration. 
    // If not found, create the default config (it's the first time).
    //
    //LOG.debug("About to load configuration");
    ClientSettings* config = ClientSettings::getInstance();


    // Set the root dir for the DMTClient.
    // All config files are stored under this location.
    StringBuffer configPath(PlatformAdapter::getHomeFolder());
    DeviceManagementNode::setConfigPath(configPath);

    config->open();
    StringBuffer infoMsg("");
    if (!config->read()) {
        firstRun = true;
        infoMsg.sprintf("Configuration not found: using default config.");
        config->createDefaultConfig();
        config->save();
    }

    //
    // Initialize LOG file.
    //
    LOG.setLogPath(LOG_FILE_PATH);
    LOG.setLogName(LOG_FILE_NAME);
    StringBuffer title(APPLICATION_NAME);
    title += " v. ";
    title += config->getSwv();
    title += " - LOG file";

    // Print the App name + version (don't reset log)
    LOG.info("\n\n%s", title.c_str());
    if (!infoMsg.empty()) {
        LOG.info("%s", infoMsg.c_str());
    }
    
    // Check if a new version has just be installed, and do proper actions.
    config->makeUpgradeActions();
    
    LogLevel level = config->getDeviceConfig().getLogLevel();
    LOG.setLevel(level);

    if (config->checkCopiedAccountFromProfile()) {
        LOG.info("SyncML profile '%s' already exists! Account data copied from it.", SYNC_PROFILE_NAME);
    }
    LOG.debug("Application private folder: %s", configPath.c_str());

    SyncSourceConfig* ssc = config->getSyncSourceConfig(SOURCE_CONTACTS_NAME);
    if (ssc) { LOG.debug("last tstamp of %s is %d", SOURCE_CONTACTS_NAME, ssc->getLast()); }
    ssc = config->getSyncSourceConfig(SOURCE_CALENDAR_NAME);
    if (ssc) { LOG.debug("last tstamp of %s is %d", SOURCE_CALENDAR_NAME, ssc->getLast()); }
    ssc = config->getSyncSourceConfig(SOURCE_NOTES_NAME);
    if (ssc) { LOG.debug("last tstamp of %s is %d", SOURCE_NOTES_NAME, ssc->getLast()); }
    
    // Goto background if started from Autostart and not the first run.
    // (1st time the app is launched by autostart)
    if ( iModel->AutoStart() && !firstRun) {
        GoToBackground();
    }
   
    // -- Commented: we need to close the app from task list pressing "c" button --
    // Sets the application's system attribute.
    // While the system attribute is set, the application cannot be closed down.
    //CEikonEnv::Static()->SetSystem(ETrue);
    
    
    // Create the view object
    TRAP(err, iAppView = CFunambolAppView::NewL(ClientRect(), this);)
    if (err) {
        LOG.error("Error initializing AppView: %d", err);
        User::Leave(err);           // Can't proceed without the View...
    }

    // Set the default IAP from config.
    LOG.debug("InitializeClient - IAP stored in config is '%s'", config->getIapName().c_str());
    FConnection::getInstance()->setIAPName(config->getIapName());
    
    // We must originate and complete an asynchronous requests
    // inside the same thread, so we save its ID.
    RThread thisThread;
    iMainThreadID = thisThread.Id();
    
    // Adds this object to the Active Scheduler
    CActiveScheduler::Add(this);

    // Init the sync engine. Beware that the engine must be initialized in the main thread.
    TRAP(err, iSyncEngine = CSyncEngine::NewL(this);)
    if (err) {
        LOG.error("Error initializing SyncEngine: %d", err);
        DisplayMessage(err, EErrorLevel);
    }

    // Create the Push engine. It DOESN'T start the CTP now.
    TRAP(err, iPushEngine = CPushEngine::NewL(this);)
    if (err) {
        LOG.error("Error initializing PushEngine: %d", err);
        DisplayMessage(err, EErrorLevel);
    }
    
    // Check if we have a pending configuration sync to perform
    if (config->getPendingSyncConfig() || config->isCredentialsEmpty()) 
    {
        if (firstRun) {
            TBool autostart = EFalse;
            if (config->getSyncUsing() != ESyncUsingManual) {
                // If PUSH or SCHEDULED, ask the user if autostart is to be enabled.
                autostart = ShowAutostartDialog();
            }
            EnableAutoStart(autostart);
        }
        
        LOG.debug("Displaying First run dialog");
        
        // We do not allow the user to quit this dialog until the
        // synchronization of the config succeeds
        changeAppTitle(R_STRING_CREDENTIALS_VERIFICATION_TITLE);
        TBool cancelled;
        
        TRAPD( err, 
        {
            iAccountSettingsDialog = CAccountSettingsDialog::NewL(R_MENUBAR_ACCOUNT_SETTINGS,
                                                                  this, cancelled);
            iAccountSettingsDialog->ExecuteLD(R_FIRST_RUN_DIALOG);
        }
        )
        iAccountSettingsDialog = NULL;
        
        if (err) {
            LOG.error("Error executing AccountSettings dialog: %d", err);
            DisplayMessage(err, EErrorLevel); 
            return;
        }
        
        // If we get here, then the sync config has been properly synchronized
        // or the user cancelled
        if (cancelled) {
            LOG.info("User cancelled the Login: exiting!");
            FConnection::getInstance()->closeConnection();
            CAknAppUi::Exit();
        }
        resetAppTitle();
    }
    
   
    // Safe check: at this point the IMAP account MUST exist.
    // If not, we recreate it.
    if (ENABLE_SOURCE_MAIL) {
        if (config->createIMAPAccount()) {
            LOG.info("IMAP account created.");
        }
    }

    // Safe check: at this point the SyncML profile MUST exist.
    // If not, we recreate it.
    if (iSyncEngine->getActiveProfileID() == -1) {
        // Leave if profile not found.
        DisplayMessage(ERR_PROFILE_NOT_FOUND, EErrorLevel);
        User::Leave(ERR_PROFILE_NOT_FOUND);
    }

    // CFunambolAppView need to be added to the control stack, 
    // to receive and manage key events.
    TRAP( err, AddToStackL(iAppView);)
    if (err) {
        LOG.error("Error adding the view to the control stack: %d", err);
        DisplayMessage(err, EErrorLevel);
    }

    // Update status text of all sources
    // Note: SyncEngine MUST be already initialized before this call!
    RefreshAllSourceTimestamps();

    CreateSortedSourcesArray();
    
    // Starts the push or scheduled service, checking the config settings.
    StartServices();
}



void CFunambolAppUi::CreateSortedSourcesArray()
{
    iSortedSourceIDs.Reset();
    
    if (ENABLE_SOURCE_CONFIG_IN_SYNCALL) {
        iSortedSourceIDs.Append(SOURCE_CONFIG);  // config is synced for first?
    }
    if (ENABLE_SOURCE_MAIL) {
        iSortedSourceIDs.Append(SOURCE_MAIL);
    }
    iSortedSourceIDs.Append(SOURCE_CONTACTS);
    iSortedSourceIDs.Append(SOURCE_CALENDAR);
    if (ENABLE_SOURCE_NOTES) {
        iSortedSourceIDs.Append(SOURCE_NOTES);
    }
}

void CFunambolAppUi::resetAppTitle() {
    changeAppTitle(R_APPLICATION_NAME);
}
 
void CFunambolAppUi::changeAppTitle(TInt resource) 
{
    TBuf<64> title;
    TRAPD(err, 
         {
            iEikonEnv->ReadResourceL(title, resource);
            CAknTitlePane* titlePane = static_cast<CAknTitlePane*>
                                      ( StatusPane()->ControlL( TUid::Uid( EEikStatusPaneUidTitle ) ) );
            titlePane->SetTextL(title);
         } 
    )
    if (err) {
        LOG.error("Error changing application title: code %d", err);
    }
}

void CFunambolAppUi::loadAppIcon()
{
    //
    // Now using the SVG image
    //
    /*TInt err = KErrNone;
    CEikStatusPane* sp = ((CAknAppUi*)iEikonEnv->EikAppUi())->StatusPane();
    if (sp) {
        CAknContextPane* cPane = NULL;
        TRAP(err, cPane = (CAknContextPane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidContext));)
        if (err) {
            LOG.error("Error getting CAknContextPane: %d", err);
            return;
        }

        if(cPane) {
            RBuf picturePath;
            picturePath.Assign(charToNewBuf(BITMAPS_MBM_PATH));
            TRAP(err, cPane->SetPictureFromFileL(picturePath, 
                                                 EMbmFunambolMainlogo, 
                                                 EMbmFunambolMainlogo_mask);)
            if (err) {
                LOG.error("Error setting picture to main icon: %d", err); 
            }
            picturePath.Close();
        }
    }*/
}


RArray<TInt> CFunambolAppUi::GetActiveSourcesList()
{
    RArray<TInt> list;
    ClientSettings* config = ClientSettings::getInstance();

    for (TInt i=0; i < iSortedSourceIDs.Count(); i++) 
    {
        StringBuffer name = getSourceNameFromID(iSortedSourceIDs[i]);
        SyncSourceConfig* ssc = config->getSyncSourceConfig(name.c_str());
        if (ssc && strcmp(ssc->getSync(), SYNC_NONE_TEXT)) {
            list.AppendL(iSortedSourceIDs[i]);
        }
    }
    
    //LOG.debug("GetActiveSourcesList: count = %d", list.Count());
    return list;
}



// -----------------------------------------------------------------------------
// CFunambolAppUi::HandleCommandL()
// Takes care of command handling.
// -----------------------------------------------------------------------------
//
void CFunambolAppUi::HandleCommandL( TInt aCommand )
{
    //LOG.debug("HandleCommandL - event received: %d", aCommand);
    
    switch( aCommand )
    {
        case ESyncAll:
        {
            LOG.debug("Starting sync of ALL");
            StartSync(ESyncTriggeredManual);
            break;
        }
        case ESync:
        {
            TInt sourceID = iAppView->GetCurrentSourceSelected();
            if (sourceID != -1) {
                LOG.debug("Starting sync source selected (%d)", sourceID);
                StartSync(sourceID, ESyncTriggeredManual);
            }
            break;
        }
        case EEnable:
        {
            TInt sourceID = iAppView->GetCurrentSourceSelected();
            if (sourceID != -1) {
                LOG.debug("Enable source %s", getSourceNameFromID(sourceID).c_str());
                iSyncEngine->setSourceSyncMode(sourceID, ESyncModeNormal);
                RefreshSourceTimestamp(sourceID);
            }
            break;
        }
        case ECancelSync:
        {
            LOG.debug("Stopping current sync");
            if (iSyncEngine) {
                iSyncEngine->StopSync();
            }
            break;
        }
        case EEikCmdExit:
        {
            // this is the case when pressing c in task list
            if (iSyncEngine && iSyncEngine->CheckSyncInProgress()) 
            {
                    iSyncEngine->StopSync();    // If sync is already finished, nothing is done
            }
            
            LOG.debug("Closing application...");
            if (iPushEngine->isPushRunning()) {
                iPushEngine->stop();
            }
            LOG.info("Application closed.");
            CAknAppUi::Exit();
            break;
        }
        case EAknSoftkeyExit:
        {
            Exit();
            break;
        }
        case EAknSoftkeyCancel:
        {
            GoToBackground();
            break;
        }
        case EAccountSettings:
        {
            ShowAccountSettingsDialog();
            break;
        }
        case ESyncSettings:
        {
            ShowSyncSettingsDialog();
            break;
        }
        case ECleanup:
        {
            ShowCleanupDialog();
            break;
        }
        case EUpdateSw:
        {
            updateSoftware();
            break;
        }
        case EHelp:
        {
            CArrayFix<TCoeHelpContext>* buf = CCoeAppUi::AppHelpContextL();
            HlpLauncher::LaunchHelpApplicationL(iEikonEnv->WsSession(), buf);
            break;
        }
        case EAbout:
        {
            ShowAboutDialog();
            break;
        }
        default:
        {
            LOG.error("HandleCommandL - Invalid command: %d", aCommand);
            //Panic( EFunambolUi );
            break;
        }
    }
}


void CFunambolAppUi::OnFireButton()
{
    TInt position = iAppView->GetCurrentItemSelected();
    LOG.debug("Fire button pressed - current position is %d", position);
    
    //
    // The "SyncAll / Cancel Sync" item
    //
    if (ENABLE_SYNC_ALL && (position == 0)) {
        if (iSyncEngine->CheckSyncInProgress()) {
            // TODO: add warning, like on Exit()
            iSyncEngine->StopSync();
        }
        else {
            StartSync(ESyncTriggeredManual);
        }
        return;
    }
    
    //
    // Other (source) items
    //
    TInt sourceID = iAppView->GetCurrentSourceSelected();
    TSyncMode syncMode = iSyncEngine->getSourceSyncMode(sourceID);
    if (syncMode != ESyncModeNone) {
        // Start the sync for that source
        StartSync(sourceID, ESyncTriggeredManual);
        return;
    }
    else {
        // Format a query message: "Do you want to enable sync of <SourceName>?"
        TBuf<64> enableText, sourceText;
        iEikonEnv->ReadResourceL(enableText, R_STRING_QUERY_ENABLE_SOURCE);
        switch (sourceID) 
        {
            case SOURCE_MAIL:
                iEikonEnv->ReadResourceL(sourceText, R_STRING_LABEL_EMAIL);
                break;
            case SOURCE_CONTACTS:
                iEikonEnv->ReadResourceL(sourceText, R_STRING_LABEL_CONTACTS);
                break;
            case SOURCE_CALENDAR:
                iEikonEnv->ReadResourceL(sourceText, R_STRING_LABEL_CALENDAR);
                break;
            case SOURCE_NOTES:
                iEikonEnv->ReadResourceL(sourceText, R_STRING_LABEL_NOTES);
                break;
        }
        const wchar_t* sourceLabel = bufToNewWchar(sourceText);
        
        // Prompt a query dialog
        TBuf<64> queryText;
        queryText.Format(enableText, sourceLabel);
        TBuf<1> dummy = _L("");
        TBool toEnable = iEikonEnv->QueryWinL(queryText, dummy);
        
        if (toEnable) {
            LOG.debug("Enable source %s", getSourceNameFromID(sourceID).c_str());
            iSyncEngine->setSourceSyncMode(sourceID, ESyncModeNormal);
            RefreshSourceTimestamp(sourceID);
        }
    }
}



// -----------------------------------------------------------------------------
//  Called by the framework when the application status pane
//  size is changed.  Passes the new client rectangle to the
//  AppView
// -----------------------------------------------------------------------------
//
void CFunambolAppUi::HandleStatusPaneSizeChange()
{
    LOG.debug("Screen size changed");
    iAppView->SetRect( ClientRect() );
} 

CArrayFix<TCoeHelpContext>* CFunambolAppUi::HelpContextL() const
{
    // TODO: add help context
    _LIT(KGeneral_Information,"General_Information");
    CArrayFixFlat<TCoeHelpContext>* array = new(ELeave)CArrayFixFlat<TCoeHelpContext>(1);
    CleanupStack::PushL(array);
    array->AppendL(TCoeHelpContext(KUidFunambolApp, KGeneral_Information));
    CleanupStack::Pop(array);
    return array;
}

void CFunambolAppUi::HandleWsEventL(const TWsEvent &aEvent, CCoeControl *aDestination)
{

    if (aEvent.Type() == KAknUidValueEndKeyCloseEvent) {
        // Handle redkey event: Go to background
        GoToBackground();
    } 
    else 
    {
        CAknAppUi::HandleWsEventL(aEvent, aDestination);
    }
}


void CFunambolAppUi::Exit() 
{
    //
    // Display a custom query message (#1): "Are you sure you want to exit?"
    //
    ClientSettings* config = ClientSettings::getInstance();
    TSyncUsing syncUsing = config->getSyncUsing();
    TInt resourceID = R_STRING_QUERY_EXIT_MANUAL;
    
    if (syncUsing == ESyncUsingPush) {
        // commented: the push could be temporarily down, but we still need
        // to warn the user...
        //if (iPushEngine->isPushRunning()) {
            resourceID = R_STRING_QUERY_EXIT_PUSH;
        //}
    }    
    else if (syncUsing == ESyncUsingScheduled) {
        resourceID = R_STRING_QUERY_EXIT_SCHEDULED;
    }
    
    
    // Display the query dialog.
    TBool toExit = iEikonEnv->QueryWinL(resourceID);
    if (toExit) 
    {
        // The user selected "Yes": will exit.
        // Check if a sync is running.
        if (iSyncEngine && iSyncEngine->CheckSyncInProgress()) 
        {
            // A sync is running: ask user to stop it (query #2)
            if (iEikonEnv->QueryWinL(R_STRING_QUERY_STOP_SYNC_TO_EXIT)) {
                iSyncEngine->StopSync();    // If sync is already finished, nothing is done
            }
            else {
                // Back to main UI
                return;
            }
        }
        
        LOG.debug("Closing application...");
        if (iPushEngine->isPushRunning()) {
            iPushEngine->stop();
        }
        LOG.info("Application closed.");
        CAknAppUi::Exit();
    }
    else 
    {
        // The user selected "No": keep the app opened.
    }
}

TBool CFunambolAppUi::CheckSyncInProgress() 
{
    if (iSyncEngine) {
        return iSyncEngine->CheckSyncInProgress();
    }
    return EFalse;
}

void CFunambolAppUi::GoToBackground()
{
    TApaTask task(iEikonEnv->WsSession());
    task.SetWgId( CEikonEnv::Static()->RootWin().Identifier());
    task.SendToBackground();
}

void CFunambolAppUi::GoToForeground()
{
    TApaTask task(iEikonEnv->WsSession());
    task.SetWgId( CEikonEnv::Static()->RootWin().Identifier());
    task.BringToForeground();
}

TBool CFunambolAppUi::isInForeground() 
{
    return iForeground;
}


void CFunambolAppUi::EnableAutoStart(const TBool aEnable)
{
    iModel->AutoStartEnable() = aEnable;
    iModel->ExternalizeL();
    
    if (aEnable) {
        LOG.info("Autostart is now enabled");
    } else {
        LOG.info("Autostart is now disabled");
    }
}

TBool CFunambolAppUi::IsAutoStartEnabled()
{
    return iModel->AutoStartEnable();
}


void CFunambolAppUi::ShowAccountSettingsDialog()
{
    if (iSyncEngine && iSyncEngine->CheckSyncInProgress()) {
        // A sync is already running
        DisplayMessage(INFO_SYNC_IN_PROGRESS, EInfoLevel);
        return;
    }
    
    
    LOG.info("Opening AccountSettings dialog");
    
    changeAppTitle(R_STRING_ACCOUNT_SETTINGS_TITLE);

    TBool cancelled;
    TRAPD(err,
    {
        iAccountSettingsDialog = CAccountSettingsDialog::NewL(R_MENUBAR_ACCOUNT_SETTINGS,
                                                              this, cancelled);
        iAccountSettingsDialog->ExecuteLD(R_ACCOUNT_SETTINGS_DIALOG);
    }
    )
    if (err) {
        LOG.error("Error executing AccountSettings dialog: %d", err);
        DisplayMessage(err, EErrorLevel); 
        return;
    }

    // Restore title and buttons
    refreshMainButtons();
    resetAppTitle();
    iAccountSettingsDialog = NULL;
    
    // Starts the push or scheduled service, checking the config settings.
    StartServices();
}

void CFunambolAppUi::ShowSyncSettingsDialog()
{
     if (iSyncEngine && iSyncEngine->CheckSyncInProgress()) {
         // A sync is already running
         DisplayMessage(INFO_SYNC_IN_PROGRESS, EInfoLevel);
         return;
     }
     
     LOG.info("Opening SyncSettings dialog");
     TInt err = KErrNone;
     CSyncSettingsDialog* dialog = NULL;

     // Create the dialog
     // Note: the menubar is not used (we have only 2 buttons), but we need to
     //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
     //       Just in case we need again a menubar in future...
     TRAP( err, dialog = CSyncSettingsDialog::NewL(R_MENUBAR_SYNC_SETTINGS, this); )
     if (err) {
         LOG.error("Error initializing SyncSettings dialog: %d", err);
         DisplayMessage(err, EErrorLevel); 
         return;
     }
     
     if (dialog) {
         // Launch the dialog (blocking). The dialog will destruct itself at exit ("D" suffix).
         TRAP( err, dialog->ExecuteLD(R_SYNC_SETTINGS_DIALOG); )
         if (err) {
             LOG.error("Error executing SyncSettings dialog: %d", err);
             DisplayMessage(err, EErrorLevel); 
             return;
         }
     }

     // Restore title and buttons
     refreshMainButtons();
     resetAppTitle();
     RefreshAllSourceTimestamps();
}

void CFunambolAppUi::ShowCleanupDialog()
{
     if (iSyncEngine && iSyncEngine->CheckSyncInProgress()) {
         // A sync is already running
         DisplayMessage(INFO_SYNC_IN_PROGRESS, EInfoLevel);
         return;
     }
     
     LOG.info("Opening Cleanup dialog");
     TInt err = KErrNone;
     CCleanupDialog* dialog = NULL;

     // Create the dialog
     // Note: the menubar is not used (we have only 2 buttons), but we need to
     //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
     //       Just in case we need again a menubar in future...
     TRAP( err, dialog = CCleanupDialog::NewL(R_MENUBAR_SYNC_SETTINGS, this); )
     if (err) {
         LOG.error("Error initializing Cleanup dialog: %d", err);
         DisplayMessage(err, EErrorLevel); 
         return;
     }

     if (dialog) {
         // Launch the dialog (blocking). The dialog will destruct itself at exit ("D" suffix).
         TRAP( err, dialog->ExecuteLD(R_CLEANUP_DIALOG); )
         if (err) {
             LOG.error("Error executing SyncSettings dialog: %d", err);
             DisplayMessage(err, EErrorLevel); 
             return;
         }
     }

     // Restore title and buttons
     refreshMainButtons();
     resetAppTitle();
}


TBool CFunambolAppUi::ShowAutostartDialog()
{
    LOG.debug("Displaying Autostart dialog");
    bool ret = ETrue;
    CAknMessageQueryDialog* dlg = new (ELeave)CAknMessageQueryDialog(); 
    dlg->PrepareLC(R_QUERY_DIALOG);
 
    HBufC* title = iEikonEnv->AllocReadResourceLC(R_AUTOSTART_DIALOG_TITLE);
    dlg->QueryHeading()->SetTextL(*title);
    CleanupStack::PopAndDestroy();     //title

    HBufC* msg = iEikonEnv->AllocReadResourceLC(R_AUTOSTART_DIALOG_TEXT);
    dlg->SetMessageTextL(*msg);
    CleanupStack::PopAndDestroy();     //msg
    
    changeMainMenuButtons(R_AVKON_SOFTKEYS_OK_CANCEL);
    
    // Display the dialog (returns the ID of button selected)
    if (dlg->RunLD() == EAknSoftkeyOk) {
        LOG.debug("User selected OK to Autostart dialog.");
        ret = ETrue;
    }
    else {
        LOG.debug("User selected Cancel to Autostart dialog.");
        ret = EFalse;
    }
    
    // Restore main menu buttons
    changeMainMenuButtons(R_MAIN_BUTTONS_SYNCALL);
    return ret;
}


void CFunambolAppUi::ShowAboutDialog()
{
    LOG.debug("Displaying About dialog");
    changeAppTitle(R_ABOUT_DIALOG_TITLE);
    
    TRAPD(err, {
               iAboutDialog = CAboutDialog::NewL(this);
               iAboutDialog->ExecuteLD(R_ABOUT_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error displaying About dialog: code %d", err);
    }
    
    // Set to NULL, to know we're not showing the about dialog.
    iAboutDialog = NULL;
    
    // Restore title and buttons
    refreshMainButtons();
    resetAppTitle();
}


void CFunambolAppUi::DynInitMenuPaneL( TInt aResourceId ,CEikMenuPane* aMenuPane )
{
    if ( aResourceId == R_MENU )
    {
        if (iSyncEngine != NULL &&
            iSyncEngine->CheckSyncInProgress())
        {
            aMenuPane->SetItemDimmed(ESync,   ETrue);
            aMenuPane->SetItemDimmed(EEnable, ETrue);
        }
        else {
            aMenuPane->SetItemDimmed(ECancelSync, ETrue);
            
            // Change dynamically the "Sync <sourceName>" menu options.
            TInt sourceID = iAppView->GetCurrentSourceSelected();
            TInt resourceID = 0;
            
            // If syncMode = "none", we set the label to "Enable <sourceName>".
            bool isEnabled = true;
            if (iSyncEngine->getSourceSyncMode(sourceID) == ESyncModeNone) {
                isEnabled= false;
                aMenuPane->SetItemDimmed(ESync, ETrue);
            }
            else {
                aMenuPane->SetItemDimmed(EEnable, ETrue);
            }
            
            switch (sourceID)
            {
                case SOURCE_MAIL:
                    if (isEnabled) resourceID = R_STRING_LABEL_SYNC_EMAIL;
                    else           resourceID = R_STRING_LABEL_ENABLE_EMAIL;
                    break;
                case SOURCE_CONTACTS:
                    if (isEnabled) resourceID = R_STRING_LABEL_SYNC_CONTACTS;
                    else           resourceID = R_STRING_LABEL_ENABLE_CONTACTS;
                    break;
                case SOURCE_CALENDAR:
                    if (isEnabled) resourceID = R_STRING_LABEL_SYNC_CALENDAR;
                    else           resourceID = R_STRING_LABEL_ENABLE_CALENDAR;
                    break;
                case SOURCE_NOTES:
                    if (isEnabled) resourceID = R_STRING_LABEL_SYNC_NOTES;
                    else           resourceID = R_STRING_LABEL_ENABLE_NOTES;
                    break;
                default:
                    resourceID = R_STRING_LABEL_SYNCALL;
                    break;
            }
            
            if (isEnabled) aMenuPane->SetItemTextL(ESync,   resourceID);
            else           aMenuPane->SetItemTextL(EEnable, resourceID);
        }
        
        if (!ENABLE_HELP) {
            aMenuPane->SetItemDimmed( EHelp, ETrue );
        }

        // Check if we must show the upgrade menu
        UpdateManager updateManager(this);
        if (updateManager.isNewVersionAvailable()) {
            aMenuPane->SetItemDimmed( EUpdateSw, EFalse);
        } else {
            aMenuPane->SetItemDimmed( EUpdateSw, ETrue);
        }
    }
}



void CFunambolAppUi::RefreshAllSourceTimestamps() 
{
    if (!iAppView) {
        return;     // Safe check
    }
    
    if (ENABLE_SOURCE_MAIL) {
        RefreshSourceTimestamp(SOURCE_MAIL);
    }
    RefreshSourceTimestamp(SOURCE_CONTACTS);
    RefreshSourceTimestamp(SOURCE_CALENDAR);
    
    if (ENABLE_SOURCE_NOTES) {
        RefreshSourceTimestamp(SOURCE_NOTES);
    }
}


void CFunambolAppUi::RefreshSourceTimestamp(const TInt aSourceID)
{
    // Safe checks
    if (aSourceID == SOURCE_CONFIG)                        { return; }
    if (aSourceID == SOURCE_NOTES && !ENABLE_SOURCE_NOTES) { return; }
    if (aSourceID == SOURCE_MAIL  && !ENABLE_SOURCE_MAIL)  { return; }
    
    TBuf<32>        statusText;
    TLastSyncStatus status;
    TTime           lastSyncTime;
    TInt            lastSyncResult = 0;
    TBool           sourceGreyed = EFalse;

    // Check if source is active
    TSyncMode syncMode = iSyncEngine->getSourceSyncMode(aSourceID);
    if (syncMode == ESyncModeNone) {
        // Source is disabled: grey it
        sourceGreyed = ETrue;
    }
    
    status = iSyncEngine->GetLastSyncStatus(aSourceID, &lastSyncTime, &lastSyncResult);
    statusText = FormatStatusText(status, lastSyncTime);
    iAppView->RefreshSourceStatus(aSourceID, statusText, sourceGreyed);
}


void CFunambolAppUi::RefreshAllSourceLastStatus() 
{
    if (!iAppView) {
        return;     // Safe check
    }
    
    if (ENABLE_SOURCE_MAIL) {
        RefreshSourceLastStatus(SOURCE_MAIL);
    }
    RefreshSourceLastStatus(SOURCE_CONTACTS);
    RefreshSourceLastStatus(SOURCE_CALENDAR);
    
    if (ENABLE_SOURCE_NOTES) {
        RefreshSourceLastStatus(SOURCE_NOTES);
    }
}

void CFunambolAppUi::RefreshSourceLastStatus(const TInt aSourceID)
{
    // Safe checks
    if (aSourceID == SOURCE_CONFIG)                        { return; }
    if (aSourceID == SOURCE_NOTES && !ENABLE_SOURCE_NOTES) { return; }
    if (aSourceID == SOURCE_MAIL && !ENABLE_SOURCE_MAIL)   { return; }
    
    TBool sourceGreyed = EFalse;

    // Check if source is active
    TSyncMode syncMode = iSyncEngine->getSourceSyncMode(aSourceID);
    if (syncMode == ESyncModeNone) {
        // Source is disabled: grey it
        sourceGreyed = ETrue;
    }
    
    iAppView->RefreshSourceBufferedStatus(aSourceID, sourceGreyed);
}


TBuf<32> CFunambolAppUi::FormatStatusText(const TLastSyncStatus aSourceStatus,
                                          const TTime aSourceLastSyncTime) 
{
    TBuf<32> statusText;

    switch (aSourceStatus)
    {
        case ELastSyncSuccessful:
            _LIT(KFormatTStamp, "%/0%1%/1%2%/2%3 %-B%:0%J%:1%T%+B");
            aSourceLastSyncTime.FormatL(statusText, KFormatTStamp);
            break;
        case ELastSyncFailed:
            iEikonEnv->ReadResourceL(statusText, R_STRING_LAST_SYNC_FAILED);
            break;
        case ELastSyncNotFound:
            iEikonEnv->ReadResourceL(statusText, R_STRING_NOT_SYNCHRONIZED);
            break;

        default:
            break;
    }
    
    return statusText;
}



void CFunambolAppUi::StartSync(const TSyncRunners from) 
{
    // Sync only active sources
    RArray<TInt> activeSources = GetActiveSourcesList();
    StartSync(activeSources, from);
    activeSources.Close();
}

void CFunambolAppUi::StartSync(const TInt aSourceID, const TSyncRunners from) 
{
    RArray<TInt> sourceIDList;
    sourceIDList.Append(aSourceID);
    StartSync(sourceIDList, from);
    sourceIDList.Close();
}


void CFunambolAppUi::StartSync(const RArray<TInt> &aSourceIDList,
                               const TSyncRunners from)
{
    iSyncEngine->StartSync(aSourceIDList, from);
}


void CFunambolAppUi::checkCredentials()
{
    // Stop the push or scheduled service, if active.
    StopServices();
   
    LOG.info("Checking user's credentials...");

    //
    // Starts the Config Sync (blocking).
    //
    StartSync(SOURCE_CONFIG, ESyncTriggeredManual);
}

bool CFunambolAppUi::StopServices()
{
    bool ret = false;
    ClientSettings* config = ClientSettings::getInstance();
    TSyncUsing syncUsing = config->getSyncUsing();
    
    if (syncUsing == ESyncUsingPush) {
        if (iPushEngine->isPushRunning()) {
            iPushEngine->stop();
            ret = true;
        }
    }    
    else if (syncUsing == ESyncUsingScheduled) {
        if (iSyncEngine->isSchedulerEnabled()) {
            iSyncEngine->disableScheduler();
            ret = true;
        }
    }
    
    return ret;
}


bool CFunambolAppUi::StartServices()
{
    bool ret = false;
    ClientSettings* config = ClientSettings::getInstance();
    TSyncUsing previousSyncUsing = config->getSyncUsing();
    
    if (previousSyncUsing == ESyncUsingPush) {
        if (!iPushEngine->isPushRunning()) 
        {
            LOG.info("Starting Push service");
            iPushEngine->start();
            
            // Autostart MUST be enabled with push service.
            EnableAutoStart(ETrue);
            ret = true;
        }
    }
    else if (previousSyncUsing == ESyncUsingScheduled) {
        // Restart scheduler if stopped
        if (!iSyncEngine->isSchedulerEnabled()) {
            int minutes = config->getSchedulerMinutes();
            LOG.info("Starting Schedule service (%d minutes)", minutes);
            iSyncEngine->enableScheduler(true, minutes * 60);
        }
        // Autostart MUST be enabled with push service.
        EnableAutoStart(ETrue);
    }
    
    return ret;
}	




void CFunambolAppUi::UpdateSourceStatus(const TInt aSourceID,
                                        const TCurrentSyncStatus aSyncStatus,
                                        const TInt aSyncData) 
{   
    TInt resourceID = -1;
    
    if (aSourceID == SOURCE_CONFIG) 
    {
        // Status of config sync won't refresh the listbox items like other sources.
        if (aSyncStatus == ESyncStatusCompleted) 
        {
            // 'aSyncData' is the result code of the sync
            iLastConfigSyncResult = aSyncData;
        }
        return;
    }

    //LOG.debug("UpdateSourceStatus");
    
    switch (aSyncStatus) 
    {
        case ESyncStatusConnecting:
            resourceID = R_STRING_SYNC_CONNECTING;
            changeMainScreenMenuButtons(R_MAIN_BUTTONS_CANCELSYNC);
            break;
        case ESyncStatusConnected:
            resourceID = R_STRING_SYNC_CONNECTED;
            break;
        case ESyncStatusDisconnected:
            resourceID = R_STRING_SYNC_DISCONNECTED;
            break;
        case ESyncStatusLoggingOn:
            resourceID = R_STRING_SYNC_LOGGING_ON;
            break;
        case ESyncStatusLoggedOn:
            resourceID = R_STRING_SYNC_LOGGED_ON;
            break;
        case ESyncStatusSendingData:
            resourceID = R_STRING_SYNC_SENDING_DATA;
            break;
        case ESyncStatusReceivingData:
            resourceID = R_STRING_SYNC_RECEIVING_DATA;
            break;
        case ESyncStatusMappings:
            resourceID = R_STRING_SYNC_MAPPINGS;
            break;
        case ESyncStatusCompleted:
        {
            //
            // Sync ended, succesfully.
            //
            resourceID = R_STRING_SYNC_COMPLETED;
            if (!iSyncEngine->CheckSyncInProgress()) {
                changeMainScreenMenuButtons(R_MAIN_BUTTONS_SYNCALL);
            }
            
            // The source has been synced, set the timestamp.
            // If timestamp was 0, next time we would have a slow sync.
            // Not necessary for Emails: we will set the timestamp in ImapEmailEngine with its real value.
            if (aSourceID != SOURCE_MAIL) {
                iSyncEngine->setSourceTimestamp(aSourceID, 1);      // Use User::TickCount() ?
            }
            
            // If push service, let's remove this source from the CP queue.
            // A new CP sync is not necessary, because this source is now in sync.
            TSyncUsing syncUsing = ClientSettings::getInstance()->getSyncUsing();
            if (syncUsing == ESyncUsingPush) {
                iSyncEngine->removeFromScheduler(aSourceID);    // Only for CP
            }
            break;
        }
        
        case ESyncStatusNotCompleted:
        case ESyncStatusError:
        {
            //
            // Sync ended, with an error.
            //
            resourceID = R_STRING_SYNC_NOT_COMPLETED;
            if (!iSyncEngine->CheckSyncInProgress()) {
                changeMainScreenMenuButtons(R_MAIN_BUTTONS_SYNCALL);
            }
            if (aSyncData) {
                //DisplayMessage(aSyncData, EErrorLevel);
            }
            break;
        }
        
        case ESyncStatusCanceled:
        {
            //
            // Sync ended, user canceled it intentionally.
            //
            resourceID = R_STRING_SYNC_CANCELED;
            changeMainScreenMenuButtons(R_MAIN_BUTTONS_SYNCALL);
            break;
        }
            
        case ESyncStatusSendingItemNumber: 
        {
            // "Sending %d...": aSyncData contains the total number of sent items
            TBuf<32> KFormat, statusText;
            iEikonEnv->ReadResourceL(KFormat, R_STRING_SYNC_SENDING_ITEM_NUMBER);
            statusText.Format(KFormat, aSyncData);
            iAppView->RefreshSourceStatus(aSourceID, statusText);
            return;
        } 
        case ESyncStatusReceivingItemNumber:
        {
            // "Receiving %d...": aSyncData contains the total number of received items
            TBuf<32> KFormat, statusText;
            iEikonEnv->ReadResourceL(KFormat, R_STRING_SYNC_RECEIVING_ITEM_NUMBER);
            statusText.Format(KFormat, aSyncData);
            iAppView->RefreshSourceStatus(aSourceID, statusText);
            return;
        }

        default:
            return; // Should not happen
    }
    
    // Updates the correspondant item text in the View
    if (resourceID != -1) {
        HBufC* statusText = iEikonEnv->AllocReadResourceLC(resourceID);
        iAppView->RefreshSourceStatus(aSourceID, *statusText);
        CleanupStack::PopAndDestroy();      // statusText
    }
    //LOG.debug("UpdateSourceStatus :: DONE");
}

void CFunambolAppUi::changeMainScreenMenuButtons(const TInt menuResourceID)
{
    if (IsMainButtonsVisible()) {
        changeMainMenuButtons(menuResourceID);
    }
}

void CFunambolAppUi::changeMainMenuButtons(const TInt menuResourceID)
{
    CEikButtonGroupContainer* cba = NULL;
    TRAPD(err, cba = CEikButtonGroupContainer::Current();)
    if (err) {
        LOG.error("changeMainMenuButtons: trapped error %d", err);
        return;
    }
    if (cba) {
        cba->SetCommandSetL(menuResourceID);
        cba->DrawNow();
    }
}

bool CFunambolAppUi::IsMainButtonsVisible()
{
    //LOG.debug("IsMainButtonsVisible");

    bool ret = false;
    HBufC* syncAllLabel    = iEikonEnv->AllocReadResourceLC(R_STRING_LABEL_SYNCALL);
    HBufC* cancelSyncLabel = iEikonEnv->AllocReadResourceLC(R_STRING_LABEL_CANCEL_SYNC);
    
    //
    // Read label texts currently displayed in the control pane (CBA) 
    // for left, middle (if in use), and right softkeys
    // http://wiki.forum.nokia.com/index.php/TSS000675_-_Retrieving_text_for_softkey_labels
    //
    CEikButtonGroupContainer* cba = CEikButtonGroupContainer::Current();
    if (cba) {
        MEikButtonGroup* buttonGroup = cba->ButtonGroup();
        for(TInt pos=0; pos<3; pos++) 
        {
            TInt cmdId = buttonGroup->CommandId(pos);
            CCoeControl* button = buttonGroup->GroupControlById(cmdId);               
            if (button && buttonGroup->IsCommandVisible(cmdId)) 
            {
                // The label is the first control owned by the softkey
                CEikLabel* label = static_cast<CEikLabel*>(button->ComponentControl(0));
                const TDesC* txt = label->Text();
                if (!txt->Compare(*syncAllLabel) || !txt->Compare(*cancelSyncLabel)) {
                    ret = true;
                    //LOG.debug("Yes, main screen");
                    break;
                }
            }
        }
    }
    
    CleanupStack::PopAndDestroy(2);     // syncAllLabel, cancelSyncLabel
    
    //if (!ret) {
    //    LOG.debug("No, other screen");
    //}
    return ret;

}

void CFunambolAppUi::refreshMainButtons()
{
    if (!iSyncEngine) { return; }
    
    if (iSyncEngine->CheckSyncInProgress()) {
        changeMainScreenMenuButtons(R_MAIN_BUTTONS_CANCELSYNC);
    }
    else {
        changeMainScreenMenuButtons(R_MAIN_BUTTONS_SYNCALL);
    }
}


void CFunambolAppUi::DisplayMessage(const TInt aMsgID, const TWarningLevel aWarningLevel) 
{
    RThread thisThread;
    TInt thisThreadID = thisThread.Id();
    if (thisThreadID != iMainThreadID) 
    {
        // It's not the main thread -> the popup won't work.
        // So we call DisplayMessage() from the Active Object ;)
        iRequestedMsgID = aMsgID;
        iRequestedWarningLevel = aWarningLevel;
        RequestAsyncAction(EDisplayMessage);
        return;
    }
    
    TBuf<MAX_POPUP_MESSAGE_SIZE> msg, KFormat;

    // Load the message from resources
    switch (aMsgID) 
    {
        case INFO_AUTOSTART_ENABLED:
            iEikonEnv->ReadResourceL(msg, R_STRING_INFO_AUTOSTART_ENABLED);
            break;
        case ERR_AUTH_FAILED:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_AUTH_FAILED);
            break;
        case ERR_NETWORK_ERROR:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_NETWORK_ERROR);
            break;
        case ERR_BAD_INPUT_CREDENTIALS:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_BAD_INPUT_CREDENTIALS);
            break;

        case INFO_SYNC_IN_PROGRESS:
            iEikonEnv->ReadResourceL(msg, R_STRING_INFO_SYNC_IN_PROGRESS);
            break;
        case ERR_CONNECTING:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_CONNECT);
            break;
        case ERR_CTP_NO_CONNECTION:
            // TODO: read '15 minutes' from config (MaxCTPRetry)
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_CTP_NO_CONNECTION);
            break;
        case ERR_INVALID_CREDENTIALS:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_INVALID_CREDENTIALS);
            break;
        case ERR_SYNCML_GENERIC_FAILURE:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_SYNCML_GENERIC_FAILURE);
            break;
        case KErrCancel:
            // Sync was cancelled by user: don't show messages!
            LOG.info("The operation was cancelled by the user.");
            return;
        case ERR_PROFILE_NOT_FOUND:
            iEikonEnv->ReadResourceL(KFormat, R_STRING_ERR_PROFILE_NOT_FOUND);
            msg.Format(KFormat, WSYNC_PROFILE_NAME);
            break;
        case ERR_ACCOUNT_NOT_FOUND:
            iEikonEnv->ReadResourceL(KFormat, R_STRING_ERR_ACCOUNT_NOT_FOUND);
            msg.Format(KFormat, WSYNC_PROFILE_NAME);
            break;
        case ERR_SOURCE_NOT_FOUND:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_SOURCE_NOT_FOUND);
            break;
        case ERR_SERVER_BUSY:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_SERVER_BUSY);
            break;
        case ERR_SERVICE_UNAVAILABLE:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_SERVICE_UNAVAILABLE);
            break;
            
        case ERR_READING_CONFIG:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_GENERIC_ERROR);  // TODO error msg
            break;

            // Should not happen
        case 0:
            return;
            
        // Not used
        case ERR_ACCOUNT_EXPIRED:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_ACCOUNT_EXPIRED);
            break;
        // Not used
        case ERR_ACCOUNT_NOT_ACTIVATED:
            iEikonEnv->ReadResourceL(msg, R_STRING_ERR_ACCOUNT_NOT_ACTIVATED);
            break;

        default:
            // Generic System error, see: http://www.newlc.com/Symbian-OS-Error-Codes.html
            iEikonEnv->ReadResourceL(KFormat, R_STRING_ERR_GENERIC_ERROR);
            msg.Format(KFormat, aMsgID);
            break;
    }
    
    
    if (isInForeground()) {
        // We are in foreground: popup a message to the user.
        msgBox(msg, aWarningLevel);
    }
    else {
        // We are in background: just log the msg.
        // TODO: here we could remember the message and display a popup when application is opened again.
        // (eventually, we could force the foreground only for some error messages...)
        StringBuffer tmp = bufToStringBuffer(msg);
        if (aWarningLevel == EErrorLevel) {
            LOG.error("(popup hidden: app is in bkground) %s", tmp.c_str());
        } else {
            LOG.info("(popup hidden: app is in bkground) %s", tmp.c_str());
        }
    }
    
}


void CFunambolAppUi::ConfigSyncEnded()
{
    ClientSettings* config = ClientSettings::getInstance();
    TBool goToMainScreen = EFalse;
    
    // iLastConfigSyncResult is set at the end of the configSync, 
    // through a callback to updateSourceStatus().
    LOG.debug("Config sync ended with code %d", iLastConfigSyncResult);
    switch (iLastConfigSyncResult)
    {
        case 0:         // OK.
        case 404:       // Configuration source not found (old server).
        {
            // TODO: case 404: disable configSyncSource to avoid synchronizing it
            //       again in future: set its syncmode = "none" for instance.
            goToMainScreen = ETrue;
            config->setPendingSyncConfig(false);
            
            // Config sync ended correctly. So we propagate the current APN selected
            // by the user into the config (and so into SyncML profile + Email account).
            LOG.debug("Setting IAP '%s' from FConnection to config", FConnection::getInstance()->getIAPName().c_str());
            config->setIapName(FConnection::getInstance()->getIAPName());
            // We must force the config to be saved
            config->forceDirtyOnMainNode();
            config->save();
            break;
        }
        case 401:
        case 402:
        case 403:
        {
            // Authentication failed.
            DisplayMessage(ERR_AUTH_FAILED, EErrorLevel);
            goToMainScreen = EFalse;
            break;
        }
        case 407:
        {
            // Invalid credentials.
            DisplayMessage(ERR_INVALID_CREDENTIALS, EErrorLevel);
            goToMainScreen = EFalse;
            break;
        }
        default:
        {
            // Other errors (network?)
            DisplayMessage(ERR_NETWORK_ERROR, EErrorLevel);
            goToMainScreen = EFalse;
            break;
        }
    }
  
    // Inform the dialog that the sync has terminated
    if (iAccountSettingsDialog) {
        iAccountSettingsDialog->CheckCredentialsEnded(goToMainScreen);
    }
}


void CFunambolAppUi::contactsCleanupCompleted() 
{
    LOG.debug("Cleanup of Contacts completed.");
    
    // Nothing to do, we wait for the calendar cleanup to be
    // finished before we can remove the waitDlg
    delete contactsManager;
    contactsManager = NULL;
    if (iLastCleanupSource == SOURCE_CONTACTS) {
        LOG.info("Cleanup completed.");
        delete cleanupWaitDlg;
        cleanupWaitDlg = NULL;
        StartServices();
    }
    
    // Reset the last timestamp: so next time it will be a slow sync.
    iSyncEngine->setSourceTimestamp(SOURCE_CONTACTS, 0);
}

void CFunambolAppUi::calendarCleanupCompleted() 
{
    LOG.debug("Cleanup of Calendar completed.");
    delete calendarManager;
    calendarManager = NULL;

    if (iLastCleanupSource == SOURCE_CALENDAR) {
        LOG.info("Cleanup completed.");
        //User::After(2 * 1000 * 1000);           // useless
        delete cleanupWaitDlg;
        cleanupWaitDlg = NULL;
        StartServices();
    }
    
    // Reset the last timestamp: so next time it will be a slow sync.
    iSyncEngine->setSourceTimestamp(SOURCE_CALENDAR, 0);
}

void CFunambolAppUi::mailCleanupCompleted() 
{
    LOG.debug("Cleanup of Emails completed.");
    delete iMailManager;
    iMailManager = NULL;

    if (iLastCleanupSource == SOURCE_MAIL) {
        LOG.info("Cleanup completed.");
        delete cleanupWaitDlg;
        cleanupWaitDlg = NULL;
        StartServices();
    }
    
    // Reset the last timestamp: so next time it will be a slow sync.
    iSyncEngine->setSourceTimestamp(SOURCE_MAIL, 0);
}


void CFunambolAppUi::notesCleanupCompleted() 
{
    LOG.debug("Cleanup of Notes completed.");
    
    // Nothing to do, we wait for the notes cleanup to be
    // finished before we can remove the waitDlg
    
    
    if (iLastCleanupSource == SOURCE_NOTES) {
        LOG.info("Cleanup completed.");
        delete cleanupWaitDlg;
        cleanupWaitDlg = NULL;
        StartServices();
    }
    
    // Reset the last timestamp: so next time it will be a slow sync.
    iSyncEngine->setSourceTimestamp(SOURCE_NOTES, 0);
}




void CFunambolAppUi::cleanup(TBool email, TBool contacts, TBool calendar,
                             TBool notes) {

    if (!email && !contacts && !calendar && !notes) {
        return;
    }

    TInt err;
    LOG.debug("Cleanup in progress");

    // Display a wait note
    cleanupWaitDlg = new ProgressDialog(false);
    cleanupWaitDlg->StartWaitNoteL();
    TBuf<128> waitText;
    iEikonEnv->ReadResourceL(waitText, R_STRING_CLEANUP_IN_PROGRESS);
    cleanupWaitDlg->UpdateProcessL(0, waitText);
 
    TRequestStatus status;

    //
    // The order of these checks must match the order of source cleanups
    // that follow
    if (email) {
        iLastCleanupSource = SOURCE_MAIL;
    }
    if (contacts) {
        iLastCleanupSource = SOURCE_CONTACTS;
    }
    if (calendar) {
        iLastCleanupSource = SOURCE_CALENDAR;
    }
    if (notes) {
        iLastCleanupSource = SOURCE_NOTES;
    }

    // We must stop the services to avoid starting a sync in the middle of the cleanup
    // Services will be restarted at the end of the cleanup
    StopServices();

    
    if (email) {
        TRAP(err,
            iMailManager = CCleanupMailManager::NewL(this);
            iMailManager->DeleteAllMailsL();
        );
    }
    
    if (contacts) {
        TRAP(err,
            contactsManager = CCleanupContactsManager::NewL(this);
            contactsManager->DeleteAllEntriesL();
        );
    }

    if (calendar) {
        TRAP(err,
            calendarManager = CCleanupCalendarManager::NewL(this);
            calendarManager->DeleteAllEntriesL();
        );
    }

    if (notes) {
        // TODO: Cleanup Notes!
        //       - Using CNotepadApi class? (how?)
        //       - Touching directly the dat file? (where is it?)
        notesCleanupCompleted();
    }
}

void CFunambolAppUi::HandleForegroundEventL(TBool aForeground)
{
    // Call Base class method
    CAknAppUi::HandleForegroundEventL(aForeground);
    
    // Copy to internal attribute, to know if we are in foreground or not.
    iForeground = aForeground;

    if(aForeground) {
        LOG.debug("Brought to foreground");
        
        if (iAccountSettingsDialog) 
        {
            LOG.debug("in account screen");
        }
        else if (iAboutDialog) 
        {
            // If here, it means that we're displaying the aboout dialog 
            // TODO: use a state var to know which screen is displayed...
            LOG.debug("in about screen");
            iAboutDialog->RefreshDialog();
        }
        else if (iSyncEngine) 
        {
            // Main screen.
            // We have gained the focus, refresh the sync timestamps
            LOG.debug("in main screen");
            if (iSyncEngine->CheckSyncInProgress()) {
                // Don't want to see the timestamp of a source that is under sync.
                // So we refresh the status with the last one received.
                RefreshAllSourceLastStatus();
            } 
            else {
                RefreshAllSourceTimestamps();
            }
            refreshMainButtons();
        }
    }
}

void CFunambolAppUi::updateSoftware()
{
    UpdateManager updateManager(this);
    updateManager.forceUpdate();
}


StringBuffer CFunambolAppUi::readStringResource(const TInt resourceID)
{
    StringBuffer ret("");
    if (resourceID <= 0) {
        return ret;     // not valid resource ID
    }
    
    // Do not TRAP this call!
    HBufC* text = iEikonEnv->AllocReadResourceLC(resourceID);
    ret = bufToStringBuffer(*text);
    CleanupStack::PopAndDestroy();      // text
    
    return ret;
}


void CFunambolAppUi::disableSyncScheduler()
{
    if (iSyncEngine) {
        iSyncEngine->disableScheduler();
    }
}


// ------------ Active Object methods --------------

void CFunambolAppUi::RunL()
{
    //LOG.debug("CFunambolAppUi::RunL - status = %d, state = %d", iStatus.Int(), iAsyncAction);
    if(iStatus == KErrNone) 
    {
        switch(iAsyncAction)
        {
            case EDisplayMessage:
            {
                DisplayMessage(iRequestedMsgID, iRequestedWarningLevel);
                break;
            }
            //
            // Add here other async actions
            //
            default:
                break;
        }
        iAsyncAction = EIdle;
    }
    else {
        LOG.error("CFunambolAppUi::RunL() error: status = %d", iStatus.Int());
    }
}

void CFunambolAppUi::DoCancel()
{
    // nothing to do
    LOG.debug("CSyncEngine DoCancel()");
}

TInt CFunambolAppUi::RunError(TInt aError) {
    LOG.error("CFunambolAppUi RunL() error: %d", aError);
    return 0;
}


void CFunambolAppUi::RequestAsyncAction(const TAsyncAction aAction)
{
    LOG.debug("Requesting async action! state is %d", aAction);
    
    // Active object will switch to the correct method to call, based on this value.
    iAsyncAction = aAction;
    
    iStatus = KRequestPending;
    if(!IsActive()) { 
        SetActive(); 
    }
    
    // Signals that the asynchronous request associated with 'iStatus' is complete
    // Note: we must do it in the same thread that originated the request,
    //       so we signal it in the main thread.
    RThread mainThread;
    mainThread.Open(iMainThreadID);

    TRequestStatus* status = &iStatus;
    mainThread.RequestComplete(status, KErrNone);
}


bool CFunambolAppUi::SetListboxFontSize(CFormattedCellListBoxItemDrawer* drawer)
{
    TRect rect = CEikonEnv::Static()->EikAppUi()->ApplicationRect();
    //LOG.debug("Screen size is %d x %d", rect.Width(), rect.Height());
    
    // Portrait screen types, with small resolution.
    if ((rect.Height() >= rect.Width()) && (rect.Width() < 300)) {
        // Reduce the listbox fon size.
        // Standard fonts are too big on small devices.
        CFormattedCellListBoxData* data = drawer->ColumnData();
        data->SetSubCellFontL(1, CEikonEnv::Static()->SymbolFont());    // LatinBold19()
        data->SetSubCellFontL(2, CEikonEnv::Static()->DenseFont());
        return true;
    }
    return false;
}





// Test the CTP 
//void CFunambolAppUi::TestCTP() 
//{
//    FConnection* connection = FConnection::getInstance();
//    if (connection) {
//        connection->setIAPDefaultName("Default");
//
//        CTPService* ctp = CTPService::getInstance();
//        ctp->startCTP();
//    }
//}
//
// Test the CP
//void CFunambolAppUi::TestCP()
//{
//    RArray<TInt> s;
//    s.Append(SOURCE_CONTACTS);
//    iSyncEngine->StartSyncL(s, ESyncTriggeredClientPush);
//}


// End of File
