/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// INCLUDE FILES
#include <coemain.h>
#include "FunambolAppView.h"

#include <eikenv.h>
#include <eikon.hrh>
#include <e32cmn.h>
#include <akniconarray.h> 
#include <AknsDrawUtils.h>
#include <AknsBasicBackgroundControlContext.h>

#include "uid3.h"
#include "base/util/symbianUtils.h"
#include "base/SymbianLog.h"
#include "commonDefs.h"
#include "customization.h"
#include "utils.h"


/// The format for each CAknDoubleLargeStyleListBox row: 
/// "<position>\t<first line>\t<second line>\t".
_LIT(KListboxFormat, "%d\t%S\t%S\t");



// -----------------------------------------------------------------------------
// CFunambolAppView::NewL()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CFunambolAppView* CFunambolAppView::NewL(const TRect& aRect, CFunambolAppUi* aAppUI)
{
    CFunambolAppView* self = CFunambolAppView::NewLC( aRect, aAppUI );
    CleanupStack::Pop( self );
    return self;
}

// -----------------------------------------------------------------------------
// CFunambolAppView::NewLC()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CFunambolAppView* CFunambolAppView::NewLC(const TRect& aRect, CFunambolAppUi* aAppUI)
{
    CFunambolAppView* self = new ( ELeave ) CFunambolAppView;
    CleanupStack::PushL( self );
    self->ConstructL( aRect, aAppUI );
    return self;
}

// -----------------------------------------------------------------------------
// CFunambolAppView::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CFunambolAppView::ConstructL(const TRect& aRect, CFunambolAppUi* aAppUI)
{
    iAppUI = aAppUI;
    
    // Create a window for this application view
    CreateWindowL();
    
    // Creates the listbox of icon+label. 
    TRAPD(err, CreateListBoxL(aRect);)
    if (err) {
        LOG.error("Error creating Listbox: code %d", err);
        User::Leave(err);   // Forward the leave
    }
    
    // Set the windows size
    SetRect( aRect );
    
    iAppUI->SetListboxFontSize(iListBox->ItemDrawer());

    iBgContext = CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain,aRect,ETrue);

    // Activate the window, which makes it ready to be drawn
    ActivateL();
}


// C++ default constructor can NOT contain any code, that might leave.
CFunambolAppView::CFunambolAppView() 
{
    iListboxNumberOfItems = 0;
    iListboxItems = NULL;
}

// Destructor.
CFunambolAppView::~CFunambolAppView()
{
    delete iBgContext;
    delete [] iListboxItems;
}


// -----------------------------------------------------------------------------
// CFunambolAppView::Draw()
// Draws the display.
// -----------------------------------------------------------------------------
//
void CFunambolAppView::Draw( const TRect& aRect ) const
{
    // Get the standard graphics context
    CWindowGc& gc = SystemGc();

    // Redraw the background using the default skin
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    MAknsControlContext* cc = AknsDrawUtils::ControlContext( this );
    AknsDrawUtils::Background( skin, cc, this, gc, aRect );

    // Gets the control's extent
    TRect drawRect( Rect() );
    

    //iListBox->SetItemHeightL(25);
    //iListBox->View()->SetItemHeight(25);

    // Clears the screen
    //gc.Clear( drawRect );

}

// -----------------------------------------------------------------------------
// CFunambolAppView::SizeChanged()
// Called by framework when the view size is changed.
// -----------------------------------------------------------------------------
//
void CFunambolAppView::SizeChanged()
{
    if(iBgContext) {
        iBgContext->SetRect(Rect());
        if ( &Window() ) {
            iBgContext->SetParentPos( PositionRelativeToScreen() );
        }
    }
    if (iListBox) {
        iListBox->SetExtent(TPoint(0,0), iListBox->MinimumSize() );
    }
    
    iAppUI->SetListboxFontSize(iListBox->ItemDrawer());
    
    DrawNow();
}



TTypeUid::Ptr CFunambolAppView::MopSupplyObject(TTypeUid aId)
{
    if (iBgContext ) {
       return MAknsControlContext::SupplyMopObject( aId, iBgContext );
    }
    return CCoeControl::MopSupplyObject(aId);
}


void CFunambolAppView::CreateListBoxL(const TRect&/* aRect*/)
{
    // Fill the iListboxItems array (dynamically).
    // This should be done only once, here.
    LOG.debug("CreateListBox");
    LoadListboxData();
    
    //
    // Create the UI listBox
    //
    if (USE_MAIN_SCREEN_BIG_ICONS) {
        iListBox = new (ELeave) CAknDoubleLargeStyleListBox();
    } else {
        iListBox = new (ELeave) CAknDoubleGraphicStyleListBox();
    }
    iListBox->SetContainerWindowL(*this);

    // Load default ListBox data from resource
    TResourceReader rr;
    iCoeEnv->CreateResourceReaderLC(rr, R_SYNC_LIST);
    iListBox->ConstructFromResourceL(rr);
    
    
    //
    // Load svg icons from MIF file, and append to ListBox
    //
    CArrayPtr<CGulIcon>* icons = new(ELeave) CAknIconArray(iListboxNumberOfItems);
    CleanupStack::PushL(icons);

    TInt notesIdx, mailIdx;
    if (ENABLE_SYNC_ALL) {
        appendIcon(icons, MAIN_LOGO_ICON_ID);
        mailIdx = 1;
        if (ENABLE_SOURCE_MAIL) {
            notesIdx = 4;
        }else {
            notesIdx = 3;
        }
    } 
    else {
        mailIdx = 0;
	    if (ENABLE_SOURCE_MAIL) {
		    notesIdx = 3;
	    }else {
	        notesIdx = 2;
	    }
    }
    
    // Normal (solid) icons
    if (ENABLE_SOURCE_MAIL) {
        appendIcon(icons, MAIL_ICON_ID);
    }
    appendIcon(icons, CONTACTS_ICON_ID);
    appendIcon(icons, CALENDAR_ICON_ID);
    if (ENABLE_SOURCE_NOTES) {
        appendIcon(icons, NOTES_ICON_ID);
    }
    
    // Dimmed (transparent) icons
    if (ENABLE_SOURCE_MAIL) {
        appendIcon(icons, MAIL_DIMMED_ICON_ID);
    }
    appendIcon(icons, CONTACTS_DIMMED_ICON_ID);
    appendIcon(icons, CALENDAR_DIMMED_ICON_ID);
    if (ENABLE_SOURCE_NOTES) {
        appendIcon(icons, NOTES_DIMMED_ICON_ID);
    }
    
    
    // Set the icons array to the listbox
    iListBox->ItemDrawer()->ColumnData()->SetIconArray(icons);
    CleanupStack::Pop();             // icons
    CleanupStack::PopAndDestroy();    // rr

    if (!ENABLE_SYNC_ALL) {
        // Remove the "Sync All" pane, that is the 1st item
        MDesCArray* textArray = iListBox->Model()->ItemTextArray();
        CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
        itemList->Delete(0);
        iListBox->HandleItemAdditionL();
    }
    if (!ENABLE_SOURCE_MAIL) {
        // Remove the "Email" pane
        MDesCArray* textArray = iListBox->Model()->ItemTextArray();
        CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
        itemList->Delete(mailIdx);
        iListBox->HandleItemAdditionL();
    }
    if (!ENABLE_SOURCE_NOTES) {
        // Remove the "Notes" pane
        MDesCArray* textArray = iListBox->Model()->ItemTextArray();
        CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
        itemList->Delete(notesIdx);
        iListBox->HandleItemAdditionL();
    }
    
    // Adds the scrollbar
    iListBox->CreateScrollBarFrameL(ETrue);
    
    iListBox->Model()->SetOwnershipType(ELbmOwnsItemArray);
    

    // Updates Listbox data with strings from resources (from iListboxItems)
    // Labels are updated, status are now empty.
    RefreshAllListboxItems();
    
    // Select the first item
    iListBox->SetCurrentItemIndex(0);
}


void CFunambolAppView::LoadListboxData() 
{
    // Index of the 'iListboxItems' array is the position: 0 = SyncAll.
    iListboxNumberOfItems = 5;
    
    if (!ENABLE_SYNC_ALL) { 
        --iListboxNumberOfItems;
    }
    if (!ENABLE_SOURCE_MAIL) { 
        --iListboxNumberOfItems;
    }
    if (!ENABLE_SOURCE_NOTES) { 
        --iListboxNumberOfItems;
    }
    iListboxItems = new ListboxItems[iListboxNumberOfItems];
    TInt idx = 0;
   
    if (ENABLE_SYNC_ALL) {
        iListboxItems[idx].sourceId = -1;     // SyncAll: no source ID!
        iCoeEnv->ReadResource(iListboxItems[idx].label, R_STRING_LABEL_SYNCALL);
        iListboxItems[idx].status = _L("");
        ++idx;
    }
    
    if (ENABLE_SOURCE_MAIL) {
	    iListboxItems[idx].sourceId = SOURCE_MAIL;
	    iCoeEnv->ReadResource(iListboxItems[idx].label, R_STRING_LABEL_EMAIL);
	    iListboxItems[idx].status = _L("");
	    ++idx;
    }
    
    iListboxItems[idx].sourceId = SOURCE_CONTACTS;
    iCoeEnv->ReadResource(iListboxItems[idx].label, R_STRING_LABEL_CONTACTS);
    iListboxItems[idx].status = _L("");
    ++idx;
    
    iListboxItems[idx].sourceId = SOURCE_CALENDAR;
    iCoeEnv->ReadResource(iListboxItems[idx].label, R_STRING_LABEL_CALENDAR);
    iListboxItems[idx].status = _L("");
    ++idx;
    
    if (ENABLE_SOURCE_NOTES) {
        iListboxItems[idx].sourceId = SOURCE_NOTES;
        iCoeEnv->ReadResource(iListboxItems[idx].label, R_STRING_LABEL_NOTES);
        iListboxItems[idx].status = _L("");
        ++idx;
    }
    
//    for (int i=0; i<iListboxNumberOfItems; i++) {
//        LOG.debug("%d - %d", i, iListboxItems[i].sourceId);
//    }
}


void CFunambolAppView::RefreshAllListboxItems() 
{
    TInt numItems = iListboxNumberOfItems;
    for (TInt i=0; i<numItems; i++) {
        RefreshItemText(i, iListboxItems[i].status);
    }
}




TInt CFunambolAppView::CountComponentControls() const
{
    return 1;
}

CCoeControl* CFunambolAppView::ComponentControl(TInt aIndex) const
{
    switch ( aIndex )
    {
        case 0:
            return iListBox;
        default:
            return NULL;
    }
}

TKeyResponse CFunambolAppView::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if ( aType != EEventKey ){
        return EKeyWasNotConsumed;
    }

//    LOG.debug("CFunambolAppView::OfferKeyEventL - key pressed: %04x", aKeyEvent.iCode);
//    _LIT(msgFormat, "key pressed: code 0x%04x, scanCode = 0x%02x");
//    TBuf<100> msg;
//    msg.Format(msgFormat, aKeyEvent.iCode, aKeyEvent.iScanCode);
//    msgBox(msg);
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3) {        // ???????? why this code ????????
        // Manage the actions on the AppUI.
        iAppUI->OnFireButton();
        return EKeyWasConsumed;
    }
    
    // Fix: refresh the main buttons when a key is pressed.
    // In case of dirty cba buttons, we can restore the correct button text.
    iAppUI->refreshMainButtons();
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to list box
        case EKeyDownArrow:
            return iListBox->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iListBox->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}



void CFunambolAppView::RefreshSourceStatus(const TInt aSourceID, const TDesC& aText, const TBool aDimmed) 
{
    // Get the corresponding position of the item in the Listbox
    TInt index = GetIndexFromSourceID(aSourceID);
    
    // Store the current status in a buffer!
    // In case the UI is not visible, this buffer will be used to
    // refreh the status, next time the app is in foreground.
    iListboxItems[index].status.Copy(aText);
    
    RefreshItemText(index, aText, aDimmed);
}

void CFunambolAppView::RefreshSourceBufferedStatus(const TInt aSourceID, const TBool aDimmed) 
{
    // Get the corresponding position of the item in the Listbox
    TInt index = GetIndexFromSourceID(aSourceID);
    
    // Use the buffered status (the last status received)
    RefreshItemText(index, iListboxItems[index].status, aDimmed); 
}


void CFunambolAppView::RefreshItemText(const TInt aItemIndex, const TDesC& aText, const TBool aDimmed) 
{
    TInt numItems = iListboxNumberOfItems;
    if (aItemIndex < 0 || aItemIndex >= numItems) {
        return;        // Out of range
    }

    iCriticalListboxText.Wait();
    
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
    
    // Calculate the icon index: [0 - iListboxNumberOfItems * 2]
    TInt iconIndex = aItemIndex;
    if (aDimmed) {
        // Dimmed icons have been loaded just after normal icons (see CreateListBoxL())
        int offset = iListboxNumberOfItems;
        if (ENABLE_SYNC_ALL) { 
            offset --;
        }
        iconIndex = aItemIndex + offset;
    }
    //LOG.debug("icon index = %d", iconIndex);

    TBuf<64> newTextLine;   // e.g. "2\tContacts\tNot Synchronized\t"
    newTextLine.Format(KListboxFormat, iconIndex, &(iListboxItems[aItemIndex].label), &aText);
    
    // Replace the desired item, with new text
    itemList->Delete(aItemIndex);
    
    TRAPD( err, {
           itemList->InsertL(aItemIndex, newTextLine);
           iListBox->HandleItemAdditionL();              // To update the listbox
           } )
    if (err) {
        LOG.error("RefreshItemText error: code %d", err);
    }
    
    iCriticalListboxText.Signal();
}


TInt CFunambolAppView::GetIndexFromSourceID(const TInt aSourceID) 
{
    for (int i=0; i<iListboxNumberOfItems; i++) {
        if (iListboxItems[i].sourceId == aSourceID) {
            return i;
        }
    }
    return 0;
}

TInt CFunambolAppView::GetCurrentItemSelected() 
{
    return iListBox->CurrentItemIndex();
}

TInt CFunambolAppView::GetCurrentSourceSelected() 
{
    TInt pos = GetCurrentItemSelected();
    if (pos >= 0 && pos < iListboxNumberOfItems) {
        return (iListboxItems[pos].sourceId);
    }
    return -1;
}


