/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <akntitle.h> 
#include <barsread.h>
#include "uid3.h"

#include "base/SymbianLog.h"
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "base/util/symbianUtils.h"
#include "base/FConnection.h"

#include "commonDefs.h"
#include "customization.h"
#include "Funambol.hrh"
#include "SyncMethodDialog.h"
#include "FunambolAppUI.h"
#include "ClientSettings.h"
#include "utils.h"

using namespace Funambol;


CSyncMethodDialog* CSyncMethodDialog::NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI)
{
    CSyncMethodDialog* self = new (ELeave)CSyncMethodDialog();
    CleanupStack::PushL(self);
    self->ConstructL(aMenuTitleResourceId, aAppUI);
    CleanupStack::Pop(self);
    return self;
}

void CSyncMethodDialog::ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI)
{
    iAppUI = aAppUI;
    iSettingList = NULL;
    CAknDialog::ConstructL(aMenuTitleResourceId);
}

CSyncMethodDialog::CSyncMethodDialog() : iSyncUsing(ESyncUsingPush),
                                         iSyncInterval(0),
                                         iAPNSelection(0)
{
}


CSyncMethodDialog::~CSyncMethodDialog()
{
    if (iSettingList) {
        delete iSettingList;
        iSettingList = NULL;
    }
}

void CSyncMethodDialog::PreLayoutDynInitL()
{
    if (!iSettingList) // if we haven't already created iSettingList
    {
        // construct control and set parent
        iSettingList = new (ELeave) CCommonOptionsSettingList();
        iSettingList->SetMopParent(this);

        // Loads data into the setting list.
        load();

        // construct control based on resource definition
        TResourceReader reader;

        // CreateResourceReaderLC will allocate a buffer to be used by
        // the TResourceReader. This buffer is pushed onto the cleanup
        // stack - not the TResourceReader itself
        iCoeEnv->CreateResourceReaderLC(reader, R_SYNC_METHOD_SETTING_LIST);
        iSettingList->ConstructFromResourceL(reader);

        // Clean up the buffer allocated above, NOT the reader itself.
        // Cannot use expected item overload of PopAndDestroy() as buffer 
        // is inaccessible.
        CleanupStack::PopAndDestroy();
        
        // Creates the APN-selection list dynamically
        HBufC* defaultAPNText = iEikonEnv->AllocReadResourceL(R_STRING_APN_ALWAYS_ASK);
        iSettingList->ConstructAPNList(defaultAPNText);
        
        // Removes the vertical scrollbar
        iSettingList->initScrollbar();

        // Activate the settings list
        iSettingList->ActivateL();
    }
    
    iAppUI->changeAppTitle(R_STRING_LABEL_SYNC_METHOD);
    RefreshDynamicItems();
    CAknDialog::PreLayoutDynInitL();
}



TBool CSyncMethodDialog::OkToExitL(TInt aButtonId)
{
    //LOG.debug("CSyncMethodDialog::OkToExitL: button %d", aButtonId);
    
    // Returning ETrue causes the form to exit
    switch (aButtonId)
    {
        case EAknSoftkeyBack:
        {
            // Save data into the config: will start/stop services if necessary...
            save();
            return ETrue;
        }
        case ESubOptionsChange:      
        {
            //if (MenuShowing()) { HideMenu(); }    not using menu
            EditCurrentItem();
            break;
        }
        // Not used...
        /*case EEikCmdExit:
        case EAknSoftkeyExit:
        {
            iAppUI->Exit();
            break;
        }*/
        default:
        {
            // Other commands forwarded to CAknDialog.
            return CAknDialog::OkToExitL(aButtonId);  
        }
    }
    return EFalse;
}


TKeyResponse CSyncMethodDialog::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType != EEventKey) {
        return EKeyWasNotConsumed;
    }
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3)          // ???????? why this code ????????
    {
        EditCurrentItem();
        return EKeyWasConsumed;
    }
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to setting list
        case EKeyDownArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}


void CSyncMethodDialog::EditCurrentItem() 
{
    // Save the actual sync-using preference.
    TSyncUsing oldSyncUsing = iSettingList->getSyncUsing();
    
    TRAPD(err, iSettingList->EditCurrentItemL();)
    if (err) {
        LOG.error("Error editing current item: code %d", err);
        return;
    }
    
    // Updates the UI, some items may be hidden.
    RefreshDynamicItems();
    
    // Display a msg telling that Autostart will be enabled.
    // This is done only if:
    //   - autostart was disabled, AND
    //   - changing the syncUsing option from manual to push/scheduled
    // NOTE: autostart will be effectively enabled when restarting the service.
    if (iAppUI->IsAutoStartEnabled() == EFalse)
    {
        TSyncUsing newSyncUsing = iSettingList->getSyncUsing();
        if (oldSyncUsing == ESyncUsingManual) {
            if (newSyncUsing == ESyncUsingPush || newSyncUsing == ESyncUsingScheduled) {
                iAppUI->DisplayMessage(INFO_AUTOSTART_ENABLED, EInfoLevel);
            }
        }
    }
}


void CSyncMethodDialog::RefreshDynamicItems()
{
    switch (iSettingList->getSyncUsing())
    {
        case ESyncUsingPush:
        {
            iSettingList->HideItem(ESettingsListSyncInterval, ETrue);
            iSettingList->HideItem(ESettingsListAutostart,    ETrue);
            break;
        }
        case ESyncUsingManual:
        {
            iSettingList->HideItem(ESettingsListSyncInterval, ETrue);
            iSettingList->HideItem(ESettingsListAutostart,    EFalse);
            break;
        }
        case ESyncUsingScheduled:
        {
            iSettingList->HideItem(ESettingsListSyncInterval, EFalse);
            iSettingList->HideItem(ESettingsListAutostart,    ETrue);
            break;
        }
    }
}



void CSyncMethodDialog::load()
{
    LOG.info("Reading SyncMethod data...");
    ClientSettings* config = ClientSettings::getInstance();
    config->open();

    // Read values from config
    iSyncUsing    = config->getSyncUsing(); 
    iSyncInterval = getSchedulerIndexFromMinutes(config->getSchedulerMinutes());
    iAPNSelection = findAPNSelectionIndex(config->getIapName());
    iAutostart    = iAppUI->IsAutoStartEnabled();       // This is not in the config
    
    //LOG.debug("Autostart is currently %s", iAutostart? "enabled":"disabled");
    //LOG.debug("Current APN position is %d", iAPNSelection);
    
    // Set values into the settinglist
    iSettingList->setSyncUsing   (iSyncUsing);
    iSettingList->setSyncInterval(iSyncInterval);
    iSettingList->setAPNSelection(iAPNSelection);
    iSettingList->setAutostart   (iAutostart);
    
    config->close();
}


TBool CSyncMethodDialog::save()
{
    // Read values inserted by the user
    TSyncUsing newSyncUsing = iSettingList->getSyncUsing();
    TInt newSyncInterval    = iSettingList->getSyncInterval();
    TInt newAPNSelection    = iSettingList->getAPNSelection();
    TBool newAutostart      = iSettingList->getAutostart();


    // Check if values changed
    if ( (newSyncUsing == iSyncUsing)       && (newSyncInterval == iSyncInterval) &&
         (newAPNSelection == iAPNSelection) && (newAutostart == iAutostart) ) {
        LOG.debug("Data unchanged -> no need to save config.");
        return EFalse;
    }
    
    // Show a wait note (process can be slow)
    ProgressDialog* progressDlg = new ProgressDialog(false);
    progressDlg->StartWaitNoteL();
    TBuf<128> noteText;
    iEikonEnv->ReadResourceL(noteText, R_STRING_INFO_PLEASE_WAIT);
    progressDlg->UpdateProcessL(0, noteText);
    
    
    LOG.info("Saving SyncMethod data...");
    ClientSettings* config = ClientSettings::getInstance();
    config->open();
    bool connectionChanged = false;
    

    // ---- APN (update all components) ----
    // Note: update APN before SyncUsing, the push service could be restarted.
    if (newAPNSelection != iAPNSelection) {
        const TDesC& text = iSettingList->GetItemText(ESettingsListAPNSelection);
        StringBuffer newIAPName = bufToStringBuffer(text);
        
        StringBuffer apnAskUserText = iAppUI->readStringResource(R_STRING_APN_ALWAYS_ASK);
        if (newIAPName == apnAskUserText) {
            newIAPName = "";
        }
        LOG.info("Changing APN to '%s'", newIAPName.c_str());
        config->setIapName(newIAPName);
        
        // Set new IAP to FConnection!
        FConnection::getInstance()->setIAPName(newIAPName);
        connectionChanged = true;
    }

    // ---- SyncUsing or scheduler minutes changed (start/stop services) ----
    if ((iSyncUsing != newSyncUsing) || (newSyncInterval != iSyncInterval))
    {
        // 1. Stop the services and (if needed) the connection
        if (iSyncUsing == ESyncUsingPush || iSyncUsing == ESyncUsingScheduled) {
            iAppUI->StopServices();
        }
        
        // 2. Close connection.
        //    Will disconnect the network when nobody else is using it.
        if (connectionChanged || (newSyncUsing != ESyncUsingPush)) {
            FConnection::getInstance()->closeConnection();
        }

        // 3. Change the config parameter with new one
        config->setSyncUsing(newSyncUsing);
        
        StringBuffer tmp("Manual");
        if (newSyncUsing == ESyncUsingPush)           { tmp = "Push"; }
        else if (newSyncUsing == ESyncUsingScheduled) { tmp = "Scheduled"; }
        LOG.info("Sync mode selected is '%s'", tmp.c_str());
        
        // 4. Update the sync interval for scheduler
        if (newSyncInterval != iSyncInterval) {
            TInt minutes = getMinutesFromSchedulerIndex(newSyncInterval);
            config->setSchedulerMinutes(minutes);
            LOG.info("Scheduler interval selected is %d minutes", minutes);
        }

        // 5. Restart the services (will check the config parameter)
        if (newSyncUsing == ESyncUsingPush || newSyncUsing == ESyncUsingScheduled) {
            iAppUI->StartServices();
            newAutostart = ETrue;       // Force autostart! User has been previously notified.
        }
    }
    else if (connectionChanged && (newSyncUsing == ESyncUsingPush)) 
    {
        // We need to stop and restart push.
        iAppUI->StopServices();
        
        FConnection* conn = FConnection::getInstance();
        conn->closeConnection();
        /* -- need some time to close the connection --
        if (conn->isConnected()) {
            LOG.debug("Could not close current connection gracefully, stopping it");
            int err = conn->stopConnection();
            if (err) {
                LOG.error("Error: cannot stop current connection (code %d)", err); 
            }
        }*/
        iAppUI->StartServices();
    }
    
    
    // ---- Autostart ----
    // Note: must be set after SyncUsing, because 'newAutostart' value can be changed there.
    if (newAutostart != iAutostart) {
        iAppUI->EnableAutoStart(newAutostart);
    }
    

    // Persist config to disk. 
    // Note: if IAP name changed, will be propagated to SyncML profile + Email account.
    config->save();
    config->close();
    
    // Close the wait note
    delete progressDlg;
    
    return ETrue;
}


TInt CSyncMethodDialog::findAPNSelectionIndex(const StringBuffer& name)
{
    LOG.debug("APN name to search = %s", name.c_str());
    if (name.empty()) {
        return 0;       // default = 0 = "Always ask"
    }
    
    // Get the list of current IAP names from FConnection.
    FConnection* connection = FConnection::getInstance();
    if (!connection) {
        LOG.error("Cannot open FConnection");
        return 0;
    }
    RArray<HBufC*> iapNames = connection->GetAllIAPNames();
    
    // Find the index of the corresponding IAP name stored in the config.
    for (TInt i=0; i<iapNames.Count(); i++) {
        StringBuffer iapName = bufToStringBuffer(*(iapNames[i]));
        if (iapName == name) {
            return i+1;     // Index 0 is the "Always Ask" label.
        }
    }

    // APN name not found: default = 0 = "Always ask"
    LOG.debug("APN '%s' not found, using 'Always ask'", name.c_str());
    return 0;
}

