/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <akntitle.h> 
#include "uid3.h"
#include <akniconarray.h>
#include <eikclbd.h>

#include "base/SymbianLog.h"
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "base/util/symbianUtils.h"

#include "commonDefs.h"
#include "Funambol.hrh"
#include "ClientSettings.h"
#include "SyncSettingsDialog.h"
#include "FunambolAppUI.h"
#include "utils.h"
#include "customization.h"

#include "EmailOptionsDialog.h"
#include "PIMOptionsDialog.h"
#include "SyncMethodDialog.h"
#include "LoggingDialog.h"

using namespace Funambol;

/// The format for each CAknDoubleLargeStyleListBox row: 
/// "<position>\t<first line>\t<second line>\t".
_LIT(KSettingsListboxFormat, "%d\t%S\t%S\t");


CSyncSettingsDialog* CSyncSettingsDialog::NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI)
{
    CSyncSettingsDialog* self = new (ELeave)CSyncSettingsDialog();
    CleanupStack::PushL(self);
    self->ConstructL(aMenuTitleResourceId, aAppUI);
    CleanupStack::Pop(self);
    return self;
}

void CSyncSettingsDialog::ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI)
{
    iAppUI   = aAppUI;
    iListBox = NULL;
    CAknDialog::ConstructL(aMenuTitleResourceId);
}

CSyncSettingsDialog::CSyncSettingsDialog()
{
    iListboxNumberOfItems = 3;		// Always: Contacts + Calendar + SyncMethod

	if (ENABLE_SOURCE_MAIL) {
	    iListboxNumberOfItems ++;
	    iContactsItemIdx = 1;
	} else {
		iContactsItemIdx = 0;
	}
	iCalendarItemIdx = iContactsItemIdx + 1;
	
	if (ENABLE_SOURCE_NOTES) {
		iListboxNumberOfItems ++;
		iNotesItemIdx   = iCalendarItemIdx + 1;
		iSyncMethodItemIdx = iNotesItemIdx + 1;
	} else {
		iSyncMethodItemIdx = iCalendarItemIdx + 1;
		iNotesItemIdx = -1;		// fake value
	}
	
	if (ENABLE_LOG_SETTINGS) {
	    iListboxNumberOfItems ++;
	    iLoggingItemIdx = iSyncMethodItemIdx + 1;
	}
	else {
	    iLoggingItemIdx = -1;   // fake value
	}
}


void CSyncSettingsDialog::PreLayoutDynInitL()
{
    CAknDialog::PreLayoutDynInitL();
    
    CreateListBoxL();
    RefreshAllListboxItems();

    iAppUI->changeAppTitle(R_STRING_SYNC_SETTINGS_TITLE);
}

void CSyncSettingsDialog::PostLayoutDynInitL()
{
    iAppUI->SetListboxFontSize(iListBox->ItemDrawer());
}


TKeyResponse CSyncSettingsDialog::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType != EEventKey) {
        return EKeyWasNotConsumed;
    }

//    LOG.debug("key pressed: %04x", aKeyEvent.iCode);
//    _LIT(msgFormat, "key pressed: code 0x%04x, scanCode = 0x%02x");
//    TBuf<100> msg;
//    msg.Format(msgFormat, aKeyEvent.iCode, aKeyEvent.iScanCode);
//    msgBox(msg);
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3)          // ???????? why this code ????????
    {
        if (!iListBox) {
            CreateListBoxL();
        }
        TInt position = iListBox->CurrentItemIndex();
        ShowOptionDialog(position);
        return EKeyWasConsumed;
    }
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to list box
        case EKeyDownArrow:
            return iListBox->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iListBox->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}



TBool CSyncSettingsDialog::OkToExitL(TInt aButtonId)
{
    //LOG.debug("CSyncSettingsDialog::OkToExitL: button %d", aButtonId);
    
    // Returning ETrue causes the form to exit
    switch (aButtonId)
    {
        case EAknSoftkeyCancel:
        case EAknSoftkeyBack:
        {
            // Exit the dialog
            return ETrue;
        }
        case ESyncSettingsOpen:      
        {
            if (!iListBox) {
                CreateListBoxL();
            }
            TInt position = iListBox->CurrentItemIndex();
            ShowOptionDialog(position);
            break;
        }
        
        // Not used...
        /*case EEikCmdExit:
        case EAknSoftkeyExit:
        {
            iAppUI->Exit();
            break;
        }*/
        default:
        {
            // Other commands forwarded to CAknDialog.
            return CAknDialog::OkToExitL(aButtonId);  
        }
    }
    return EFalse;
}



void CSyncSettingsDialog::CreateListBoxL()
{
    iListBox = static_cast <CAknDoubleLargeStyleListBox*>(ControlOrNull(ESyncSettingsListbox));
    if (!iListBox) {
        LOG.error("Error getting Listbox resource!");
        TryExitL(EAknSoftkeyBack);  // calls OkToExitL()
    }
    
    // Get the text array. Only mandatory items are already in the array.
    // Optional items are iserted in the array now.
    TInt err = KErrNone;
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
    
    // Real labels will be replaced in RefreshAllListboxItems().
    _LIT(dummyLabel, "");
    
    CArrayPtr<CGulIcon>* icons = new(ELeave) CAknIconArray(iListboxNumberOfItems);
    CleanupStack::PushL(icons);
    
    //
    // Fill the listbox: labels and icons from MIF file
    //
    if (ENABLE_SOURCE_MAIL) 
    {
        TRAP(err, itemList->InsertL(0, dummyLabel);)
        appendIcon(icons, MAIL_SETTINGS_ICON_ID);
    }
    appendIcon(icons, CONTACTS_SETTINGS_ICON_ID);
    appendIcon(icons, CALENDAR_SETTINGS_ICON_ID);
    if (ENABLE_SOURCE_NOTES) 
    {
        TRAP(err, itemList->InsertL(iNotesItemIdx, dummyLabel);)
        appendIcon(icons, NOTES_SETTINGS_ICON_ID);
    }
    appendIcon(icons, SYNC_METHOD_ICON_ID);
    
    if (ENABLE_LOG_SETTINGS)
    {
        TRAP(err, itemList->InsertL(iLoggingItemIdx, dummyLabel);)
        appendIcon(icons, LOGGING_ICON_ID);
    }
    
    if (err) {
        LOG.error("Error inserting items in itemList array (code %d)", err);
        TryExitL(EAknSoftkeyBack);  // calls OkToExitL()
    }
    
    TRAP(err, iListBox->HandleItemAdditionL();)
    if (err) {
        LOG.error("Error in HandleItemAdditionL (code %d)", err);
        TryExitL(EAknSoftkeyBack);  // calls OkToExitL()
    }
    
    //
    // Set the icons array to the listbox
    //
    iListBox->ItemDrawer()->ColumnData()->SetIconArray(icons);
    CleanupStack::Pop();              // icons
    
    iListBox->CreateScrollBarFrameL(ETrue);
    iListBox->ScrollBarFrame()->SetScrollBarVisibilityL( CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto );
    
    // Select the first item
    iListBox->SetCurrentItemIndex(0);
}




void CSyncSettingsDialog::ShowOptionDialog(const TInt aPane)
{
    // If menubar is opened, we want to close it before 
    // displaying the sub-dialog.
    if (MenuShowing()) {
        HideMenu();
    }
    
    if      (aPane == iContactsItemIdx)   { ShowContactsOptionsDialog(); }
    else if (aPane == iCalendarItemIdx)   { ShowCalendarOptionsDialog(); }
    else if (aPane == iNotesItemIdx)      { ShowNotesOptionsDialog();    }
    else if (aPane == iSyncMethodItemIdx) { ShowSyncMethodDialog();      }
    else if (aPane == iLoggingItemIdx)    { ShowLoggingDialog();         }
    else if (aPane == 0)                  { ShowEmailOptionsDialog();    }
    else {
    	LOG.debug("ShowOptionDialog Error: invalid index: %d", aPane);
    }
    
    // Restore the main title.
    iAppUI->changeAppTitle(R_STRING_SYNC_SETTINGS_TITLE);
}


void CSyncSettingsDialog::ShowEmailOptionsDialog()
{
    LOG.debug("Opening Email options dialog");
    
    TRAPD( err, 
               {
               // Note: the menubar is not used (we have only 2 buttons), but we need to
               //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
               //       Just in case we need again a menubar in future...
               CEmailOptionsDialog* dialog = CEmailOptionsDialog::NewL(R_MENUBAR_SUB_OPTIONS, iAppUI);
               dialog->ExecuteLD(R_SUB_OPTIONS_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error executing EmailOptions dialog: %d", err);
        iAppUI->DisplayMessage(err, EErrorLevel); 
        return;
    }
    
    RefreshEmailStatus();
}

void CSyncSettingsDialog::ShowContactsOptionsDialog()
{
    LOG.debug("Opening Contacts options dialog");
    
    TRAPD( err, 
               {
               // Note: the menubar is not used (we have only 2 buttons), but we need to
               //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
               //       Just in case we need again a menubar in future...
               CPIMOptionsDialog* dialog = CPIMOptionsDialog::NewL(R_MENUBAR_SUB_OPTIONS, iAppUI, SOURCE_CONTACTS);
               dialog->ExecuteLD(R_SUB_OPTIONS_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error executing ContactsOptions dialog: %d", err);
        iAppUI->DisplayMessage(err, EErrorLevel); 
        return;
    }
    
    RefreshContactsStatus();
}

void CSyncSettingsDialog::ShowCalendarOptionsDialog()
{
    LOG.debug("Opening Calendar options dialog");
    
    TRAPD( err, 
               {
               // Note: the menubar is not used (we have only 2 buttons), but we need to
               //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
               //       Just in case we need again a menubar in future...
               CPIMOptionsDialog* dialog = CPIMOptionsDialog::NewL(R_MENUBAR_SUB_OPTIONS, iAppUI, SOURCE_CALENDAR);
               dialog->ExecuteLD(R_SUB_OPTIONS_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error executing CalendarOptions dialog: %d", err);
        iAppUI->DisplayMessage(err, EErrorLevel); 
        return;
    }
    
    RefreshCalendarStatus();
}

void CSyncSettingsDialog::ShowNotesOptionsDialog()
{
    LOG.debug("Opening Notes options dialog");
    
    TRAPD( err, 
               {
               // Note: the menubar is not used (we have only 2 buttons), but we need to
               //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
               //       Just in case we need again a menubar in future...
               CPIMOptionsDialog* dialog = CPIMOptionsDialog::NewL(R_MENUBAR_SUB_OPTIONS, iAppUI, SOURCE_NOTES);
               dialog->ExecuteLD(R_SUB_OPTIONS_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error executing NotesOptions dialog: %d", err);
        iAppUI->DisplayMessage(err, EErrorLevel); 
        return;
    }
    
    RefreshNotesStatus();
}

void CSyncSettingsDialog::ShowSyncMethodDialog()
{
    LOG.debug("Opening Sync Method dialog");
    
    TRAPD( err, 
               {
               // Note: the menubar is not used (we have only 2 buttons), but we need to
               //       keep it to use a CAknDialog object (if removed, we have to use CEikDialog)
               //       Just in case we need again a menubar in future...
               CSyncMethodDialog* dialog = CSyncMethodDialog::NewL(R_MENUBAR_SUB_OPTIONS, iAppUI);
               dialog->ExecuteLD(R_SUB_OPTIONS_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error executing SyncMethod dialog: %d", err);
        iAppUI->DisplayMessage(err, EErrorLevel); 
        return;
    }
    
    RefreshSyncMethodStatus();
}

void CSyncSettingsDialog::ShowLoggingDialog()
{
    LOG.debug("Opening Logging dialog");
    
    TRAPD( err, 
               {
               CLoggingDialog* dialog = CLoggingDialog::NewL(R_MENUBAR_LOGGING, iAppUI);
               dialog->ExecuteLD(R_LOGGING_DIALOG);
               }
    )
    if (err) {
        LOG.error("Error executing Logging dialog: %d", err);
        iAppUI->DisplayMessage(err, EErrorLevel); 
        return;
    }
    
    RefreshLoggingStatus();
}


void CSyncSettingsDialog::RefreshAllListboxItems() 
{
    //LOG.debug("CSyncSettingsDialog::RefreshAllListboxItems");

    RefreshContactsStatus();
    RefreshCalendarStatus();
    RefreshSyncMethodStatus();
    
	if (ENABLE_SOURCE_MAIL) {
		RefreshEmailStatus();
	}
    if (ENABLE_SOURCE_NOTES) {
    	RefreshNotesStatus();
    }
    if (ENABLE_LOG_SETTINGS) {
        RefreshLoggingStatus();
    }
}


void CSyncSettingsDialog::RefreshEmailStatus() 
{
    StringBuffer status("");
    composeSourceSettingsStatus(SOURCE_MAIL, status);
    RefreshItemText(0, status);							// Email (if used) is the first
}
    
void CSyncSettingsDialog::RefreshContactsStatus() 
{
    StringBuffer status("");
    composeSourceSettingsStatus(SOURCE_CONTACTS, status);
    RefreshItemText(iContactsItemIdx, status);
}

void CSyncSettingsDialog::RefreshCalendarStatus() 
{
    StringBuffer status("");
    composeSourceSettingsStatus(SOURCE_CALENDAR, status);
    RefreshItemText(iCalendarItemIdx, status);
}

void CSyncSettingsDialog::RefreshNotesStatus() 
{
    StringBuffer status("");
    composeSourceSettingsStatus(SOURCE_NOTES, status);
    RefreshItemText(iNotesItemIdx, status);
}

void CSyncSettingsDialog::RefreshSyncMethodStatus() 
{
    StringBuffer status("");
    composeSyncMethodStatus(status);
    RefreshItemText(iSyncMethodItemIdx, status);
}

void CSyncSettingsDialog::RefreshLoggingStatus() 
{
    StringBuffer status("");
    composeLoggingStatus(status);
    RefreshItemText(iLoggingItemIdx, status);
}

void CSyncSettingsDialog::RefreshItemText(const TInt aItemIndex, const StringBuffer& aText) 
{
    RBuf text;
    text.Assign(stringBufferToNewBuf(aText));
    RefreshItemText(aItemIndex, text);
    text.Close();
}

void CSyncSettingsDialog::RefreshItemText(const TInt aItemIndex, const TDesC& aText) 
{
    TInt numItems = iListboxNumberOfItems;
    if (aItemIndex < 0 || aItemIndex >= numItems) {
        return;        // Out of range
    }
      
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);

    // Get the label from resource
    TInt labelResourceID = getSettingsLabelResourceID(aItemIndex);
    if (labelResourceID == KErrNotFound) return; 
    HBufC* label = iEikonEnv->AllocReadResourceLC(labelResourceID);

    // e.g. "1\tContacts\tEnabled - Normal\t"
    TBuf<64> newTextLine;
    newTextLine.Format(KSettingsListboxFormat, aItemIndex, label, &aText);
    CleanupStack::PopAndDestroy();      // label
    
    // Replace the desired item, with new text
    itemList->Delete(aItemIndex);
    
    TRAPD( err, {
                 itemList->InsertL(aItemIndex, newTextLine);
                 iListBox->HandleItemAdditionL();              // To update the listbox
                } )
    if (err) {
        LOG.error("RefreshItemText error: code %d", err);
    }
}



void CSyncSettingsDialog::composeSourceSettingsStatus(const TInt aSourceID, StringBuffer& status)
{
    StringBuffer sourceName = getSourceNameFromID(aSourceID);
    
    // Get the right SyncSourceConfig
    ClientSettings* config = ClientSettings::getInstance();
    SyncSourceConfig* ssConfig = config->getSyncSourceConfig(sourceName.c_str());
    if (!ssConfig) {
        LOG.error("Config not found for source '%s'", sourceName.c_str());
        status = "";
        return;
    }
    
    //
    // Read the source sync type.
    //
    StringBuffer syncType = ssConfig->getSync();
    if ( syncType.empty() || (syncType == SYNC_NONE_TEXT) ) 
    {
        // Just return "Disabled".
        status = iAppUI->readStringResource(R_STRING_SOURCE_DISABLED);
        return;
    }
    else 
    {
        // The status returned will be "Enabled - <syncmode>".
        status = iAppUI->readStringResource(R_STRING_SOURCE_ENABLED);
        
        if (aSourceID != SOURCE_MAIL) {     // Email doesn't have a synctype to choose.
            status += " - ";
            
            if (syncType == SYNC_TWO_WAY_TEXT) {
                status += iAppUI->readStringResource(R_STRING_SYNC_TYPE_NORMAL);
            }
            else if (syncType == SYNC_ONE_WAY_FROM_CLIENT_TEXT) {
                status += iAppUI->readStringResource(R_STRING_SYNC_TYPE_TO_SERVER_ONLY);
            }
            else {
                status += iAppUI->readStringResource(R_STRING_SYNC_TYPE_TO_PHONE_ONLY);
            }
        }
    }
}


void CSyncSettingsDialog::composeSyncMethodStatus(StringBuffer& status)
{
    ClientSettings* config = ClientSettings::getInstance();
    TSyncUsing syncUsing = config->getSyncUsing();
    
    switch (syncUsing)
    {
        case ESyncUsingPush:
        {
            status = iAppUI->readStringResource(R_STRING_MODE_PUSH_SHORT);
            break;
        }
        
        case ESyncUsingManual:
        {
            status = iAppUI->readStringResource(R_STRING_MODE_MANUAL_SHORT);
            status += " - ";
            
            if (iAppUI->IsAutoStartEnabled()) {
                status += iAppUI->readStringResource(R_STRING_AUTOSTART_ON);
            }
            else {
                status += iAppUI->readStringResource(R_STRING_AUTOSTART_OFF);
            }
            break;
        }
        
        case ESyncUsingScheduled:
        {
            status = iAppUI->readStringResource(R_STRING_MODE_SCHEDULED_SHORT);
            status += " - ";
            
            TInt minutes = config->getSchedulerMinutes();
            TInt resourceID = getSchedulerResourceIDFromMinutes(minutes);
            status += iAppUI->readStringResource(resourceID);
            break;
        }
    }
    
    // Append the APN name.
    StringBuffer apnName(config->getIapName());
    if (apnName.empty()) {
        apnName = iAppUI->readStringResource(R_STRING_APN_ALWAYS_ASK);
    }
    status += " - \"";
    status += apnName;
    status += "\"";
}

void CSyncSettingsDialog::composeLoggingStatus(StringBuffer& status)
{
    ClientSettings* config = ClientSettings::getInstance();


    // Read values from config and set values into the settinglist
    LogLevel logLevel = config->getDeviceConfig().getLogLevel();

    TInt resourceID = 0;
    if (logLevel == LOG_LEVEL_NONE) { 
        resourceID = R_STRING_LOG_LEVEL_ERROR; 
    }
    else if (logLevel == LOG_LEVEL_DEBUG) { 
        resourceID = R_STRING_LOG_LEVEL_DEBUG; 
    }
    else { 
        resourceID = R_STRING_LOG_LEVEL_INFO; 
    }
    
    status = iAppUI->readStringResource(resourceID);
}



TInt CSyncSettingsDialog::getSettingsLabelResourceID(const TInt aItemIndex)
{
    //LOG.debug("CSyncSettingsDialog::getSettingsLabelResourceID - index = %d", aItemIndex);
    
	if      (aItemIndex == iContactsItemIdx)   { return R_STRING_LABEL_CONTACTS;    }
	else if (aItemIndex == iCalendarItemIdx)   { return R_STRING_LABEL_CALENDAR;    }
	else if (aItemIndex == iNotesItemIdx)      { return R_STRING_LABEL_NOTES;       }
	else if (aItemIndex == iSyncMethodItemIdx) { return R_STRING_LABEL_SYNC_METHOD; }
	else if (aItemIndex == iLoggingItemIdx)    { return R_STRING_LABEL_LOGGING;     }
	else if (aItemIndex == 0)                  { return R_STRING_LABEL_EMAIL;       }
	else {
        LOG.error("Internal error: getSettingsLabelResourceID wrong index %d", aItemIndex);
        return KErrNotFound;
	}
}
