/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <e32base.h>
#include <eikenv.h>
#include <eikdef.h>
#include <avkon.hrh>
#include <apgcli.h>
#include <aknmessagequerydialog.h>
#include <aknnotewrappers.h>
#include <stringloader.h>
#include <eikmenup.h>
#include <eikbtgpc.h>               // to change main menu buttons

#include "uid3.h"
#include "Funambol.hrh"
#include "Funambol.pan"
#include "customization.h"

#include "http/URL.h"
#include "updater/Updater.h"
#include "base/Log.h"
#include "FunambolAppUi.h"
#include "ClientSettings.h"
#include "UpgradeDialog.h"
#include "UpdateManager.h"


using namespace Funambol;

Updater*       UpdateManager::updater       = NULL;
UpdaterConfig* UpdateManager::updaterConfig = NULL;
TBool          UpdateManager::displayed     = EFalse;

Updater* UpdateManager::getUpdaterInstance() 
{
    if (!updater && !updaterConfig) {
        StringBuffer component("sym");
        const DeviceConfig& deviceConfig = ClientSettings::getInstance()->getDeviceConfig();
        StringBuffer version(deviceConfig.getSwv());
        
        // root = "Funambol/"
        StringBuffer root(APPLICATION_URI);
        root += "/";
        
        // config passed but owned by UpdateManager
        updaterConfig = new UpdaterConfig(root);
        updater       = new Updater(component, version, *updaterConfig);
    }
    return updater;
}

UpdateManager::UpdateManager(CFunambolAppUi* appUi) : appUi(appUi)
{
}

void UpdateManager::dispose()
{
    delete updater;
    delete updaterConfig;
}

void UpdateManager::checkForUpdates() {
    Updater* updater = getUpdaterInstance();
    updater->setUI(this);
    updater->start();
}

void UpdateManager::forceUpdate() {
    Updater* updater = getUpdaterInstance();
    updater->setUI(this);
    updater->forceUpdate();
}

int32_t UpdateManager::askConfirmationForMandatoryUpgrade(const UpdaterConfig& config) {
    return askConfirmationForUpgrade(config, ETrue);
}

int32_t UpdateManager::askConfirmationForUpgrade(const UpdaterConfig& config) {

    return askConfirmationForUpgrade(config, EFalse);
}


int32_t UpdateManager::askConfirmationForUpgrade(const UpdaterConfig& config,
                                                 TBool mandatory)
{
    int32_t ret;

    if (displayed) {
        // The previos message has not been dismissed yet, there is no
        // point in showing another one
        return -1;
    }

    CUpgradeDialog* dialog;
    TInt choice;

    displayed = ETrue;
    TRAPD( err, 
        {
            dialog = CUpgradeDialog::NewL(appUi, mandatory, choice);
            dialog->ShowLD();
        }
    )
    displayed = EFalse;
    if (err) {
        LOG.error("Error executing UpgradeDialog dialog: %d", err);
        // Show the user the broswer to indicate a new version is available,
        // hopefully the new version will fix this bug ;)
        return 0;
    }

    // Display the dialog (returns the ID of button selected)
    switch (choice) {
        case ENow:
            LOG.debug("User selected Now to update dialog.");
            ret = 0;
            break;
        case ELater:
            LOG.debug("User selected Later to update dialog.");
            ret = 1;
            break;
        case ESkip:
            LOG.debug("User selected Skip to update dialog.");
            ret = 2;
            break;
    }
    return ret;
}


void UpdateManager::startUpgrade(const UpdaterConfig& config) {

    const StringBuffer& url = config.getUrlUpdate();
    LOG.debug("Starting upgrade at address: %s", url.c_str());
    HBufC* targetUrl = stringBufferToNewBuf(url);
    CleanupStack::PushL(targetUrl);

    RApaLsSession apaLsSession;
    // The browser has different ids on 3rd MR and FP1. We try both
    // hoping one will work.
    const TUid KOSSBrowserUidValue[] = {0x10008D39, 0x1020724D};
    TBool found = EFalse;

    for(TUint i=0;i< sizeof(KOSSBrowserUidValue)/sizeof(TUid);++i) {
        TUid id(KOSSBrowserUidValue[i]);
        TApaTaskList taskList(CEikonEnv::Static()->WsSession());
        TApaTask task = taskList.FindApp(id);
        if(task.Exists()) {
            task.BringToForeground();
            HBufC8* param8 = HBufC8::NewLC(targetUrl->Length());
            param8->Des().Append(*targetUrl);
            task.SendMessage(TUid::Uid(0), *param8); // UID not used
            CleanupStack::PopAndDestroy(param8);
            found = ETrue;
            break;
        } else {
            HBufC* param = HBufC::NewLC(targetUrl->Length() + 2);
            param->Des().Append(_L("4 "));
            param->Des().Append(*targetUrl);
            RApaLsSession appArcSession;
            // connect to AppArc server 
            User::LeaveIfError(appArcSession.Connect()); 
            TThreadId id;
            appArcSession.StartDocument(*param,KOSSBrowserUidValue[i],id);
            appArcSession.Close(); 
            CleanupStack::PopAndDestroy(param);
            found = ETrue;
            break;
        }
    }
    if (!found) {
        LOG.error("Cannot open browser for completing update");
        // TODO: shall we show an error to the user?
    }
    CleanupStack::PopAndDestroy(targetUrl);
}

bool UpdateManager::isNewVersionAvailable() {
    Updater* updater = getUpdaterInstance();
    return updater->isNewVersionAvailable();
}


void UpdateManager::setUpdateURL(const StringBuffer& syncURL)
{
    // Get the host & port info
    URL url;
    url.setURL(syncURL.c_str());
    StringBuffer port(":80");
    if (url.port != 0) { 
        port = ":"; 
        port += itow(url.port); 
    }
    
    // Compose the url
    StringBuffer updateUrl(url.protocol);
    updateUrl += "://"; 
    updateUrl += url.host;
    updateUrl += port;
    updateUrl += UPDATE_SOFTWARE_URL_RESOURCE;
    
    LOG.debug("New update url = %s", updateUrl.c_str());
    
    // If first time, this will create the new Updater & UpdaterConfig instances.
    getUpdaterInstance();
    if (updaterConfig) {
        updaterConfig->setUrlUpdate(updateUrl);
        updaterConfig->save();
    }
}
