/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "utils.h"
#include "commonDefs.h"
#include "base/globalsdef.h"
#include "base/FConnection.h"
#include "base/SymbianLog.h"
#include "base/util/stringUtils.h"
#include "uid3.h"
#include "customization.h"

using namespace Funambol;


TInt getSourceIDFromName(const StringBuffer& name)
{
    if (name == SOURCE_CONTACTS_NAME) {
        return SOURCE_CONTACTS;
    }
    else if (name == SOURCE_CALENDAR_NAME) {
        return SOURCE_CALENDAR;
    }
    else if (name == SOURCE_NOTES_NAME) {
        return SOURCE_NOTES;
    }
    else if (name == SOURCE_MAIL_NAME) {
        return SOURCE_MAIL;
    }
    else if (name == SOURCE_BRIEFCASE_NAME) {
        return SOURCE_BRIEFCASE;
    }
    else if (name == SOURCE_CONFIG_NAME) {
        return SOURCE_CONFIG;
    }
    return -1;  // Not found
}

StringBuffer getSourceNameFromID(const TInt aSourceID)
{
    StringBuffer name("");
    switch (aSourceID)
    {
        case SOURCE_CONTACTS:
            name = SOURCE_CONTACTS_NAME;
            break;
        case SOURCE_CALENDAR:
            name = SOURCE_CALENDAR_NAME;
            break;
        case SOURCE_NOTES:
            name = SOURCE_NOTES_NAME;
            break;
        case SOURCE_MAIL:
            name = SOURCE_MAIL_NAME;
            break;
        case SOURCE_BRIEFCASE:
            name = SOURCE_BRIEFCASE_NAME;
            break;
        case SOURCE_CONFIG:
            name = SOURCE_CONFIG_NAME;
            break;
    }
    return name;
}


TInt getSchedulerIndexFromMinutes(const TInt minutes)
{
    if (minutes <= 15)       { return 0; }
    else if (minutes <= 30)  { return 1; }
    else if (minutes <= 60)  { return 2; }
    else if (minutes <= 120) { return 3; }
    else if (minutes <= 240) { return 4; }
    else if (minutes <= 480) { return 5; }
    else if (minutes <= 720) { return 6; }
    else                     { return 7; }
}

TInt getSchedulerResourceIDFromMinutes(const TInt minutes)
{
    if (minutes <= 15)       { return R_STRING_15_MIN; }
    else if (minutes <= 30)  { return R_STRING_30_MIN; }
    else if (minutes <= 60)  { return R_STRING_1_HOUR; }
    else if (minutes <= 120) { return R_STRING_2_HOUR; }
    else if (minutes <= 240) { return R_STRING_4_HOUR; }
    else if (minutes <= 480) { return R_STRING_8_HOUR; }
    else if (minutes <= 720) { return R_STRING_12_HOUR; }
    else                     { return R_STRING_1_DAY; }
}

TInt getSchedulerResourceIDFromIndex(const TInt index)
{
    switch (index)
    {
        case 0:  return R_STRING_15_MIN;
        case 1:  return R_STRING_30_MIN;
        case 2:  return R_STRING_1_HOUR;
        case 3:  return R_STRING_2_HOUR;
        case 4:  return R_STRING_4_HOUR;
        case 5:  return R_STRING_8_HOUR;
        case 6:  return R_STRING_12_HOUR;
        case 7:  return R_STRING_1_DAY;
        default: return R_STRING_15_MIN;
    }
}


TInt getMinutesFromSchedulerIndex(const TInt index)
{
    switch (index)
    {
        case 0:  return 15;
        case 1:  return 30;
        case 2:  return 60;
        case 3:  return 120;
        case 4:  return 240;
        case 5:  return 480;
        case 6:  return 720;
        case 7:  return 1440;
        default: return 15;
    }
}



bool startConnection(const TInt maxNumRetries) 
{
    FConnection* connection = FConnection::getInstance();
    if (connection->isConnected() == false) {
        if (connection->startConnection()) 
        {
            LOG.debug("Error in startConnection: retry asking APN to the user.");
            int numretries = 1;
            while (connection->startConnection("")) 
            {
                LOG.debug("Error in startConnection: retry (%d) asking APN to the user.", numretries);
                if (numretries == maxNumRetries) {
                    LOG.error("Connection error: %d attempts failed.", numretries);
                    return false;
                }
                numretries ++;
            }
        }
    }
    return true;
}

void appendIcon(CArrayPtr<CGulIcon>* icons, const TInt bmpID, TInt maskID)
{
    if (!maskID) {
        maskID = bmpID + 1;     // The mask is usally the next one
    }
    CFbsBitmap *bitmap, *mask;
    
    RBuf iconsPath;
    iconsPath.Assign(charToNewBuf(LOCALISABLE_RESOURCE_FILE_MIF));
    
    // Please do not trap here...
    AknIconUtils::CreateIconLC(bitmap, mask, iconsPath, bmpID, maskID);
    icons->AppendL(CGulIcon::NewL(bitmap, mask)); 
    
    CleanupStack::Pop(2);   // bitmap, mask
    iconsPath.Close();
}


int getBuildNumberFromVersion(const char* swv) {

    int major=0, minor=0, build=0;
    if (!swv) {
        return 0;
    }
    sscanf(swv, "%d.%d.%d", &major, &minor, &build);
    return (major*10000 + minor*100 + build);
}

int mergeRArray(RArray<TInt>& destArray, const RArray<TInt>& sourceArray)
{
    for (TInt i=0; i<sourceArray.Count(); i++)
    {
        if (destArray.Find((TInt)sourceArray[i]) == KErrNotFound) {
            destArray.Append((TInt)sourceArray[i]);
        }
    }
    return destArray.Count();
}

